/*****************************************************************************
 * BitStreamWriter.cpp: Write to Bitstreams
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

// BitStreamWriter.cpp: implementation of the BitStreamWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <memory.h>
#include "BitStreamWriter.h"



/*
 * --------------------------------------------------------------
	Constructor

	you can change the m_bitbuffer_size to any value.
	set m_bitbuffer_size = 1 for very poor performance :-)
 * --------------------------------------------------------------
 */

BitStreamWriter::BitStreamWriter()
{
	m_id = NULL;
	m_bitbuffer_size = 1;
	m_bitbuffer = new unsigned char[m_bitbuffer_size];
}

/*
 * --------------------------------------------------------------
	Destructor
 * --------------------------------------------------------------
 */
BitStreamWriter::~BitStreamWriter()
{
	Close();
	if (m_bitbuffer!=NULL)
	{
		delete m_bitbuffer;
		m_bitbuffer = NULL;
	}
}
/*
 * --------------------------------------------------------------
	Write <count> bits from <value>

	for example:
	WriteBits(0x8E,4) write 0xE to disk

 * --------------------------------------------------------------
 */
void BitStreamWriter::WriteBits(int value,int count)
{
	for (int i=count-1;i>=0;i--)
	{
		int mask = 1<<i;
		if ((value & mask)!=0)
		{
			WriteBit(1);
		}
		else
		{
			WriteBit(0);
		}
	}
}
/*
 * --------------------------------------------------------------
	Write One bit to disk
	beware: 0 <= value <= 1

	m_bitpos start to 7 and decrease to -1
	when m_bitpos is equal to -1 we go to the next byte (m_bytepos)
	of the m_bitbuffer.
	when m_bytepos is at the end of the buffer,
	the buffer is saved to disk.

    when you call Close(), the last bit will be also flushed to disk.

 * --------------------------------------------------------------
 */
void BitStreamWriter::WriteBit(int value)
{
	int mask = 0;
	if (value==1)
	{
		mask = 1<<m_bitpos;
	}
	m_bitbuffer[m_bytepos] |= mask;

	m_bitpos--;
	if (m_bitpos==-1)
	{
		m_bitpos = 7;
		m_bytepos++;
		if (m_bytepos==m_bitbuffer_size)
		{
			SaveBuffer();
		}
	}
}
/*
 * --------------------------------------------------------------
	Open a bitstream. Existing file will be removed.

	Previous stream is closed.
 * --------------------------------------------------------------
 */
bool BitStreamWriter::Open(char *filename)
{
	Close();
	m_id = fopen(filename,"wb");
	if (m_id!=NULL)
	{
		ResetBitStream();
		return true;
	}
	else
	{
		return false;
	}
}
/*
 * --------------------------------------------------------------
	Close the bitstream

	remaining bits in the internal buffer are saved to disk
	(not all the buffer !)
 * --------------------------------------------------------------
 */
void BitStreamWriter::Close()
{
	if (m_bitpos!=7)
	{
		SaveBuffer();
	}
	if (m_id!=NULL)
	{
		fclose(m_id);
		m_id=NULL;
	}

}

/*
 * --------------------------------------------------------------
	Prepare the bitstream internal state to the first write.
 * --------------------------------------------------------------
 */

void BitStreamWriter::ResetBitStream()
{
	m_bitpos = 7;
	m_bytepos = 0;
	ResetBitBuffer();
}
/*
 * --------------------------------------------------------------
	flush the internal buffer m_bitbuffer to disk
 * --------------------------------------------------------------
 */
void BitStreamWriter::SaveBuffer()
{
	if (m_id!=NULL)
	{
		int rest = 0;
		if (m_bitpos!=7)
		{
			rest = 1;
		}
		fwrite(m_bitbuffer,1,m_bytepos+rest,m_id);
		ResetBitStream();
	}
}
/*
 * --------------------------------------------------------------
	clear the content of the buffer
 * --------------------------------------------------------------
 */
void BitStreamWriter::ResetBitBuffer()
{
	memset(m_bitbuffer,0,m_bitbuffer_size);
}
