/*****************************************************************************
 * Block.cpp: Read and Write VLC Block Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "block.h"

Block::Block(void)
{
}

Block::~Block(void)
{
}

VLC Block::bstream;

// DCT Coefficient Table Zero Indices - ISO 13818-2 Table B-14
const char Block::DCT_CoefIDX0[12] = { 0, 3, 6, 17, 21, 25, 26, 34, 50, 66, 82, 98 };

// DCT Coefficient Table Zero Indices - ISO 13818-2 Table B-14 (NOTE 3-4, modified version)
const char Block::DCT_CoefIDX0m[12] = { 0, 2, 5, 16, 20, 24, 25, 33, 49, 65, 81, 97 };

// DCT Coefficient Table Zero - ISO 13818-2 Table B-14 (VLC part)
const VLC_ENTRY Block::DCT_CoefVLC0[128] =
{ { 0, 1, 1   },
  { 0, 2, 2   },
  { 0, 2, 3   },
  { 1, 2, 3   },
  { 1, 3, 4   },
  { 1, 3, 5   },
  { 2, 3, 5   },
  { 2, 3, 7   },
  { 2, 3, 6   },
  { 2, 6, 38  },
  { 2, 6, 33  },
  { 2, 6, 37  },
  { 2, 6, 36  },
  { 2, 6, 39  },
  { 2, 6, 35  },
  { 2, 6, 34  },
  { 2, 6, 32  },
  { 3, 3, 6   },
  { 3, 3, 7   },
  { 3, 3, 5   },
  { 3, 3, 4   },
  { 4, 3, 6   },
  { 4, 3, 4   },
  { 4, 3, 7   },
  { 4, 3, 5   },
  { 5, 1, 1   },
  { 6, 4, 10  },
  { 6, 4, 12  },
  { 6, 4, 11  },
  { 6, 4, 15  },
  { 6, 4, 9   },
  { 6, 4, 14  },
  { 6, 4, 13  },
  { 6, 4, 8   },
  { 7, 5, 29  },
  { 7, 5, 24  },
  { 7, 5, 19  },
  { 7, 5, 16  },
  { 7, 5, 27  },
  { 7, 5, 20  },
  { 7, 5, 28  },
  { 7, 5, 18  },
  { 7, 5, 30  },
  { 7, 5, 21  },
  { 7, 5, 17  },
  { 7, 5, 31  },
  { 7, 5, 26  },
  { 7, 5, 25  },
  { 7, 5, 23  },
  { 7, 5, 22  },
  { 8, 5, 26  },
  { 8, 5, 25  },
  { 8, 5, 24  },
  { 8, 5, 23  },
  { 8, 5, 22  },
  { 8, 5, 21  },
  { 8, 5, 20  },
  { 8, 5, 19  },
  { 8, 5, 18  },
  { 8, 5, 17  },
  { 8, 5, 16  },
  { 8, 5, 31  },
  { 8, 5, 30  },
  { 8, 5, 29  },
  { 8, 5, 28  },
  { 8, 5, 27  },
  { 9, 5, 31  },
  { 9, 5, 30  },
  { 9, 5, 29  },
  { 9, 5, 28  },
  { 9, 5, 27  },
  { 9, 5, 26  },
  { 9, 5, 25  },
  { 9, 5, 24  },
  { 9, 5, 23  },
  { 9, 5, 22  },
  { 9, 5, 21  },
  { 9, 5, 20  },
  { 9, 5, 19  },
  { 9, 5, 18  },
  { 9, 5, 17  },
  { 9, 5, 16  },
  { 10, 5, 24 },
  { 10, 5, 23 },
  { 10, 5, 22 },
  { 10, 5, 21 },
  { 10, 5, 20 },
  { 10, 5, 19 },
  { 10, 5, 18 },
  { 10, 5, 17 },
  { 10, 5, 16 },
  { 10, 5, 31 },
  { 10, 5, 30 },
  { 10, 5, 29 },
  { 10, 5, 28 },
  { 10, 5, 27 },
  { 10, 5, 26 },
  { 10, 5, 25 },
  { 11, 5, 19 },
  { 11, 5, 18 },
  { 11, 5, 17 },
  { 11, 5, 16 },
  { 11, 5, 20 },
  { 11, 5, 26 },
  { 11, 5, 25 },
  { 11, 5, 24 },
  { 11, 5, 23 },
  { 11, 5, 22 },
  { 11, 5, 21 },
  { 11, 5, 31 },
  { 11, 5, 30 },
  { 11, 5, 29 },
  { 11, 5, 28 },
  { 11, 5, 27 } };

// DCT Coefficient Table Zero - ISO 13818-2 Table B-14 (settings part)
const BLOCK_ENTRY Block::DCT_Coef0[128] =
{ { 0, 1   },
  { -1, -1 },
  { 0, 1   },
  { 1, 1   },
  { 0, 2   },
  { 2, 1   },
  { 0, 3   },
  { 3, 1   },
  { 4, 1   },
  { 0, 5   },
  { 0, 6   },
  { 1, 3   },
  { 3, 2   },
  { 10, 1  },
  { 11, 1  },
  { 12, 1  },
  { 13, 1  },
  { 1, 2   },
  { 5, 1   },
  { 6, 1   },
  { 7, 1   },
  { 0, 4   },
  { 2, 2   },
  { 8, 1   },
  { 9, 1   },
  { -2, -1 },
  { 0, 7   },
  { 1, 4   },
  { 2, 3   },
  { 4, 2   },
  { 5, 2   },
  { 14, 1  },
  { 15, 1  },
  { 16, 1  },
  { 0, 8   },
  { 0, 9   },
  { 0, 10  },
  { 0, 11  },
  { 1, 5   },
  { 2, 4   },
  { 3, 3   },
  { 4, 3   },
  { 6, 2   },
  { 7, 2   },
  { 8, 2   },
  { 17, 1  },
  { 18, 1  },
  { 19, 1  },
  { 20, 1  },
  { 21, 1  },
  { 0, 12  },
  { 0, 13  },
  { 0, 14  },
  { 0, 15  },
  { 1, 6   },
  { 1, 7   },
  { 2, 5   },
  { 3, 4   },
  { 5, 3   },
  { 9, 2   },
  { 10, 2  },
  { 22, 1  },
  { 23, 1  },
  { 24, 1  },
  { 25, 1  },
  { 26, 1  },
  { 0, 16  },
  { 0, 17  },
  { 0, 18  },
  { 0, 19  },
  { 0, 20  },
  { 0, 21  },
  { 0, 22  },
  { 0, 23  },
  { 0, 24  },
  { 0, 25  },
  { 0, 26  },
  { 0, 27  },
  { 0, 28  },
  { 0, 29  },
  { 0, 30  },
  { 0, 31  },
  { 0, 32  },
  { 0, 33  },
  { 0, 34  },
  { 0, 35  },
  { 0, 36  },
  { 0, 37  },
  { 0, 38  },
  { 0, 39  },
  { 0, 40  },
  { 1, 8   },
  { 1, 9   },
  { 1, 10  },
  { 1, 11  },
  { 1, 12  },
  { 1, 13  },
  { 1, 14  },
  { 1, 15  },
  { 1, 16  },
  { 1, 17  },
  { 1, 18  },
  { 6, 3   },
  { 11, 2  },
  { 12, 2  },
  { 13, 2  },
  { 14, 2  },
  { 15, 2  },
  { 16, 2  },
  { 27, 1  },
  { 28, 1  },
  { 29, 1  },
  { 30, 1  },
  { 31, 1  } };

// DCT Coefficient Table One Indices - ISO 13818-2 Table B-15
const char Block::DCT_CoefIDX1[12] = { 0, 15, 18, 29, 33, 37, 38, 43, 53, 65, 81, 97 };

// DCT Inverse Coefficient Table Zero [run + 2][level - 1] -> index for DCT_CoefVLC0 - from ISO 13818-2 Table B-14
// be careful using this table for non-intra blocks! -> run 0 level 1 [2][0] must be modified from 2 to 0
const int Block::DCT_Coef0INV[34][40] =
{ { 25, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  2,  4,  6, 21,  9, 10, 26, 34, 35, 36, 37, 50, 51, 52, 53, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90 },
  {  3, 17, 11, 27, 38, 54, 55, 91, 92, 93, 94, 95, 96, 97, 98, 99,100,101, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  5, 22, 28, 39, 56, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  7, 12, 40, 57, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  8, 29, 41, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 18, 30, 58, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 19, 42,102, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 20, 43, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 23, 44, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 24, 59, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 13, 60, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 14,103, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 15,104, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 16,105, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 31,106, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 32,107, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 33,108, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 45, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 46, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 47, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 48, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 49, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 61, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 62, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 63, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 64, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 65, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {109, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {110, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {111, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {112, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {113, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 } };

// DCT Coefficient Table One - ISO 13818-2 Table B-15 (VLC part)
const VLC_ENTRY Block::DCT_CoefVLC1[128] =
{ { 0, 2, 2   },
  { 0, 3, 6   },
  { 0, 5, 28  },
  { 0, 5, 29  },
  { 0, 7, 120 },
  { 0, 7, 121 },
  { 0, 7, 122 },
  { 0, 7, 123 },
  { 0, 7, 124 },
  { 0, 8, 252 },
  { 0, 8, 253 },
  { 0, 8, 250 },
  { 0, 8, 251 },
  { 0, 8, 254 },
  { 0, 8, 255 },
  { 1, 2, 2   },
  { 1, 3, 6   },
  { 1, 3, 7   },
  { 2, 3, 5   },
  { 2, 3, 7   },
  { 2, 3, 6   },
  { 2, 6, 38  },
  { 2, 6, 33  },
  { 2, 6, 37  },
  { 2, 6, 36  },
  { 2, 6, 39  },
  { 2, 6, 35  },
  { 2, 6, 34  },
  { 2, 6, 32  },
  { 3, 3, 6   },
  { 3, 3, 7   },
  { 3, 3, 5   },
  { 3, 3, 4   },
  { 4, 3, 6   },
  { 4, 3, 4   },
  { 4, 3, 7   },
  { 4, 3, 5   },
  { 5, 1, 1   },
  { 6, 3, 4   },
  { 6, 3, 5   },
  { 6, 3, 7   },
  { 6, 4, 13  },
  { 6, 4, 12  },
  { 7, 5, 28  },
  { 7, 5, 18  },
  { 7, 5, 30  },
  { 7, 5, 21  },
  { 7, 5, 17  },
  { 7, 5, 31  },
  { 7, 5, 26  },
  { 7, 5, 25  },
  { 7, 5, 23  },
  { 7, 5, 22  },
  { 8, 5, 22  },
  { 8, 5, 21  },
  { 8, 5, 20  },
  { 8, 5, 19  },
  { 8, 5, 18  },
  { 8, 5, 17  },
  { 8, 5, 16  },
  { 8, 5, 31  },
  { 8, 5, 30  },
  { 8, 5, 29  },
  { 8, 5, 28  },
  { 8, 5, 27  },
  { 9, 5, 31  },
  { 9, 5, 30  },
  { 9, 5, 29  },
  { 9, 5, 28  },
  { 9, 5, 27  },
  { 9, 5, 26  },
  { 9, 5, 25  },
  { 9, 5, 24  },
  { 9, 5, 23  },
  { 9, 5, 22  },
  { 9, 5, 21  },
  { 9, 5, 20  },
  { 9, 5, 19  },
  { 9, 5, 18  },
  { 9, 5, 17  },
  { 9, 5, 16  },
  { 10, 5, 24 },
  { 10, 5, 23 },
  { 10, 5, 22 },
  { 10, 5, 21 },
  { 10, 5, 20 },
  { 10, 5, 19 },
  { 10, 5, 18 },
  { 10, 5, 17 },
  { 10, 5, 16 },
  { 10, 5, 31 },
  { 10, 5, 30 },
  { 10, 5, 29 },
  { 10, 5, 28 },
  { 10, 5, 27 },
  { 10, 5, 26 },
  { 10, 5, 25 },
  { 11, 5, 19 },
  { 11, 5, 18 },
  { 11, 5, 17 },
  { 11, 5, 16 },
  { 11, 5, 20 },
  { 11, 5, 26 },
  { 11, 5, 25 },
  { 11, 5, 24 },
  { 11, 5, 23 },
  { 11, 5, 22 },
  { 11, 5, 21 },
  { 11, 5, 31 },
  { 11, 5, 30 },
  { 11, 5, 29 },
  { 11, 5, 28 },
  { 11, 5, 27 } };

// DCT Coefficient Table One - ISO 13818-2 Table B-15 (settings part)
const BLOCK_ENTRY Block::DCT_Coef1[128] =
{ { 0, 1   },
  { 0, 2   },
  { 0, 4   },
  { 0, 5   },
  { 9, 1   },
  { 1, 3   },
  { 10, 1  },
  { 0, 8   },
  { 0, 9   },
  { 2, 3   },
  { 4, 2   },
  { 0, 12  },
  { 0, 13  },
  { 0, 14  },
  { 0, 15  },
  { 1, 1   },
  { -1, -1 },
  { 0, 3   },
  { 2, 1   },
  { 3, 1   },
  { 1, 2   },
  { 3, 2   },
  { 11, 1  },
  { 12, 1  },
  { 13, 1  },
  { 1, 4   },
  { 0, 10  },
  { 0, 11  },
  { 1, 5   },
  { 4, 1   },
  { 5, 1   },
  { 0, 6   },
  { 0, 7   },
  { 6, 1   },
  { 7, 1   },
  { 2, 2   },
  { 8, 1   },
  { -2, -1 },
  { 5, 2   },
  { 14, 1  },
  { 15, 1  },
  { 16, 1  },
  { 2, 4   },
  { 3, 3   },
  { 4, 3   },
  { 6, 2   },
  { 7, 2   },
  { 8, 2   },
  { 17, 1  },
  { 18, 1  },
  { 19, 1  },
  { 20, 1  },
  { 21, 1  },
  { 1, 6   },
  { 1, 7   },
  { 2, 5   },
  { 3, 4   },
  { 5, 3   },
  { 9, 2   },
  { 10, 2  },
  { 22, 1  },
  { 23, 1  },
  { 24, 1  },
  { 25, 1  },
  { 26, 1  },
  { 0, 16  },
  { 0, 17  },
  { 0, 18  },
  { 0, 19  },
  { 0, 20  },
  { 0, 21  },
  { 0, 22  },
  { 0, 23  },
  { 0, 24  },
  { 0, 25  },
  { 0, 26  },
  { 0, 27  },
  { 0, 28  },
  { 0, 29  },
  { 0, 30  },
  { 0, 31  },
  { 0, 32  },
  { 0, 33  },
  { 0, 34  },
  { 0, 35  },
  { 0, 36  },
  { 0, 37  },
  { 0, 38  },
  { 0, 39  },
  { 0, 40  },
  { 1, 8   },
  { 1, 9   },
  { 1, 10  },
  { 1, 11  },
  { 1, 12  },
  { 1, 13  },
  { 1, 14  },
  { 1, 15  },
  { 1, 16  },
  { 1, 17  },
  { 1, 18  },
  { 6, 3   },
  { 11, 2  },
  { 12, 2  },
  { 13, 2  },
  { 14, 2  },
  { 15, 2  },
  { 16, 2  },
  { 27, 1  },
  { 28, 1  },
  { 29, 1  },
  { 30, 1  },
  { 31, 1  } };


// DCT Inverse Coefficient Table One [run + 2][level - 1] -> index for DCT_CoefVLC1 - from ISO 13818-2 Table B-15
const int Block::DCT_Coef1INV[34][40] =
{ { 37, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 16, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  0,  1, 17,  2,  3, 31, 32,  7,  8, 26, 27, 11, 12, 13, 14, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89 },
  { 15, 20,  5, 25, 28, 53, 54, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,100, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 18, 35,  9, 42, 55, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 19, 21, 43, 56, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 29, 10, 44, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 30, 38, 57, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 33, 45,101, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 34, 46, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 36, 47, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  4, 58, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {  6, 59, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 22,102, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 23,103, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 24,104, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 39,105, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 40,106, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 41,107, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 48, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 49, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 50, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 51, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 52, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 60, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 61, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 62, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 63, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  { 64, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {108, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {109, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {110, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {111, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 },
  {112, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 } };

// DCT DC Size Luminance Table   - ISO 13818-2 Table B-12 (VLC part)
const VLC_SIMPLEENTRY Block::DCT_DC_Size_LumaVLC[12] =
{ { 2, 0   },
  { 2, 1   },
  { 3, 4   },
  { 3, 5   },
  { 3, 6   },
  { 4, 14  },
  { 5, 30  },
  { 6, 62  },
  { 7, 126 },
  { 8, 254 },
  { 9, 510 },
  { 9, 511 } };

// DCT DC Size Luminance Table   - ISO 13818-2 Table B-12 (number part)
const char Block::DCT_DC_Size_Luma[12] =
{
	1,
	2,
	0,
	3,
	4,
	5,
	6,
	7,
	8,
	9,
	10,
	11
};

// DCT DC Size Chrominance Table - ISO 13818-2 Table B-13 (VLC part)
const VLC_SIMPLEENTRY Block::DCT_DC_Size_ChromaVLC[12] =
{ { 2, 0     },
  { 2, 1     },
  { 2, 2     },
  { 3, 6     },
  { 4, 14    },
  { 5, 30    },
  { 6, 62    },
  { 7, 126   },
  { 8, 254   },
  { 9, 510   },
  { 10, 1022 },
  { 10, 1023 } };

// DCT DC Size Chrominance Table - ISO 13818-2 Table B-13 (number part)
const char Block::DCT_DC_Size_Chroma[12] =
{
	0,
	1,
	2,
	3,
	4,
	5,
	6,
	7,
	8,
	9,
	10,
	11
};


// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 0)
const unsigned char Block::QuantizerScaleMap0[32] =
{
	2,
	4,
	6,
	8,
	10,
	12,
	14,
	16,
	18,
	20,
	22,
	24,
	26,
	28,
	30,
	32,
	34,
	36,
	38,
	40,
	42,
	44,
	46,
	48,
	50,
	52,
	54,
	56,
	58,
	60,
	62
};

// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 1)
const unsigned char Block::QuantizerScaleMap1[32] =
{
	1,
	2,
	3,
	4,
	5,
	6,
	7,
	8,
	10,
	12,
	14,
	16,
	18,
	20,
	22,
	24,
	28,
	32,
	36,
	40,
	44,
	48,
	52,
	56,
	64,
	72,
	80,
	88,
	96,
	104,
	112
};

void Block::SetType(int type)
{
	this->type = type;
}

int Block::AttachBuffer(unsigned char *buffer, int length, int bit_startpos)
{

	// init parameters
	pBuffer = buffer;
	bufferSize = INT_MAX / 4 /*length*/;

	// attach a bitstream to the buffer
	bstream.AttachBuffer(pBuffer, bufferSize + (bit_startpos + 7) / 8);
	bstream.Seek(bit_startpos);

	int val, bitSize = 0;

	int dct_dc_differential = 0;

	int begin_idx = 0;

	// if intra coded block then read dc differential value
	if(intra)
	{
		if(type == LUMA)
		{
			// first read size of diff
			val = bstream.ReadSimpleVLC((VLC_SIMPLEENTRY *) DCT_DC_Size_LumaVLC);
			// check whether value is in bounds
			_ASSERTX(val >= 0 && val < 12);
			dct_dc_size = DCT_DC_Size_Luma[val];
			bitSize += DCT_DC_Size_LumaVLC[val].digits + dct_dc_size;
		}
		else
		{
			// first read size of diff
			val = bstream.ReadSimpleVLC((VLC_SIMPLEENTRY *) DCT_DC_Size_ChromaVLC);
			// check whether value is in bounds
			_ASSERTX(val >= 0 && val < 12);
			dct_dc_size = DCT_DC_Size_Chroma[val];
			bitSize += DCT_DC_Size_ChromaVLC[val].digits + dct_dc_size;
		}
		dct_dc_size_vlc_idx = val;
		// read diff
		dct_dc_differential = bstream.ReadBits(dct_dc_size);
//		Coef[0] = dct_dc_differential;
		bl_coefBuffer[0] = dct_dc_differential;
		begin_idx = 1;
	}

	bitSize += ReadBlock(begin_idx);

	testBitSize = bitSize;

	return bitSize;
}

// returns DC coefficient
int Block::GetDCCoef(int prev_val)
{
//	int dct_diff = Coef[0];
	int dct_diff = bl_coefBuffer[0];

	if(dct_dc_size > 0)
	{
		int half_range = 1 << (dct_dc_size - 1);
		if(dct_diff < half_range) 
		{
			dct_diff += 1 - (half_range << 1);
		}
	}

	return prev_val + dct_diff;
}

int Block::WriteDataBuffer(unsigned char *buffer, int bit_startpos)
{
	int quantizer_scale;
	int *coefTable;

	// attach a bitstream to the buffer (deactivate limitation as we do not use the same buffer as for reading)
	bstream.AttachBuffer(buffer, 0x7FFFFFFF);
	bstream.Seek(bit_startpos);

	// determine which quantizer scale to use
	if(qScaleType) quantizer_scale = QuantizerScaleMap1[quantizerScaleCode - 1];
	else		   quantizer_scale = QuantizerScaleMap0[quantizerScaleCode - 1];

	// determine which table to use for VLC decoding
	// either B-14 or B-15
	if(intraVLCFormat && intra)
	{
		bstream.table = (VLC_ENTRY *) DCT_CoefVLC1;
		bstream.indices = (unsigned char *) DCT_CoefIDX1;
	}
	else
	{
		bstream.table = (VLC_ENTRY *) DCT_CoefVLC0;
		bstream.indices = (unsigned char *) DCT_CoefIDX0;
	}

	// determine which (inverse) table to use for VLC encoding
	// either B-14 or B-15
	if(intraVLCFormat && intra)	coefTable = (int *) DCT_Coef1INV;
	else						coefTable = (int *) DCT_Coef0INV;

	int length = 0;

	int dct_dc_differential = 0;

	int begin_idx = 0;

	// if intra coded block then read dc differential value
	if(intra)
	{
		if(type == LUMA)
		{
			// write size of diff
			length += bstream.WriteSimpleVLC((VLC_SIMPLEENTRY *) DCT_DC_Size_LumaVLC, dct_dc_size_vlc_idx);
		}
		else
		{
			// write size of diff
			length += bstream.WriteSimpleVLC((VLC_SIMPLEENTRY *) DCT_DC_Size_ChromaVLC, dct_dc_size_vlc_idx);
		}
//		bstream.WriteBits(Coef[0], dct_dc_size);
		bstream.WriteBits(bl_coefBuffer[0], dct_dc_size);
		length += dct_dc_size;
		begin_idx = 1;
	}

	int n;
	int level = 0;
	int run = 0;
	int block_empty = 1 - begin_idx;
	for(n = begin_idx; n < blockEOF; n++)
	{
		// get run
//		run += rCoef[n];
		run += bl_runBuffer[n];

		// half quantize
		int sign = 0;
		if(intra == 0) 
		{
//			if(Coef[n] > 0) sign = 1;
//			else if(Coef[n] < 0) sign = -1;
			if(bl_coefBuffer[n] > 0) sign = 1;
			else if(bl_coefBuffer[n] < 0) sign = -1;
		}

		// hope quantizer scale will never be 0
		_ASSERTX(quantizer_scale != 0);
//		level = (Coef[n] / quantizer_scale - sign) / 2;
		level = (bl_coefBuffer[n] / quantizer_scale - sign) / 2;

		int negative = (level >= 0) ? 1 : -1;
		if(level < 0) level = -level;

		if(level == 0) run++;
		else
		{
			// write coefficient VLC
			if(level < 41 && (run + 2) < 34) 
			{
				// if VLC exists
				if(coefTable[(run + 2) * 40 + level - 1] != -1)
				{
					if(intra == 0 && n == 0 && run == 0 && level == 1) length += bstream.WriteVLC(0);
					else											   length += bstream.WriteVLC(coefTable[(run + 2) * 40 + level - 1]);
					// write sign bit
					bstream.WriteBit(negative == -1 ? 1 : 0);
					length++;
				}
				// else use escape coding
				else
				{
					// write escape
					if(negative == -1) level = -level;

					length += bstream.WriteVLC(coefTable[0]) + 18;
//					length += bstream.WriteVLC(coefTable[1][0]) + 18;
					bstream.WriteBits(run, 6);
					// offset level to right sign;
					if(level < 0) level += 2 * 2048;
					bstream.WriteBits(level, 12);
				}
			}
			// else use escape coding
			else
			{
				// write escape
				if(negative == -1) level = -level;

				length += bstream.WriteVLC(coefTable[0]) + 18;
//				length += bstream.WriteVLC(coefTable[1][0]) + 18;
				bstream.WriteBits(run, 6);
				// offset level to right sign;
				if(level < 0) level += 2 * 2048;
				bstream.WriteBits(level, 12);
			}

			// reset run
			run = 0;
			block_empty = 0;
		}
	}

//	_ASSERTX(run < 64);
	// if block is empty
//	if(run == blockEOF) 
	if(block_empty == 1)
	{
		return 0;
	}

	// write end of block
	length += bstream.WriteVLC(coefTable[40]);
//	length += bstream.WriteVLC(coefTable[0][0]);

	// debug
//	_ASSERTX(testBitSize == length);
//	_ASSERTX(length <= testBitSize);

/*#ifdef _DEBUG
	BitStream srcBStream;
			  srcBStream.AttachBuffer(pBuffer, 2000000);
			  srcBStream.Seek(bit_startpos);
	BitStream destBStream;
			  destBStream.AttachBuffer(buffer, 2000000);
			  destBStream.Seek(bit_startpos);

    int val;
	for(int i = 0; i < length / 16; i++)
	{
		val = srcBStream.ReadBits(16);
		_ASSERTX(destBStream.ReadBits(16) == val);
	}

	val = srcBStream.ReadBits(length % 16);
	_ASSERTX(destBStream.ReadBits(length % 16) == val);

#endif*/

	//_ASSERT(length <= testBitSize);

	return length;
	
}


int Block::ReadBlock(int begin_idx)
{

	int bitSize = 0;

	int val, n = begin_idx, negative, level, run;
	BLOCK_ENTRY *pEntry;
	int quantizer_scale;

	// determine which quantizer scale to use
	if(qScaleType) quantizer_scale = QuantizerScaleMap1[quantizerScaleCode - 1];
	else		   quantizer_scale = QuantizerScaleMap0[quantizerScaleCode - 1];

	// determine which table to use for VLC decoding
	// either B-14 or B-15
	if(intraVLCFormat && intra)
	{
		bstream.table = (VLC_ENTRY *) DCT_CoefVLC1;
		bstream.indices = (unsigned char *) DCT_CoefIDX1;
	}
	else
	{
		bstream.table = (VLC_ENTRY *) DCT_CoefVLC0;
		bstream.indices = (unsigned char *) DCT_CoefIDX0;
	}

//	while(bstream.TestNextBits(6, 4) == 0) { bstream.ReadBit(); bitSize++; }

	// special case when intra block and using VLC0 table 
	// n will always be 1, so it will not be set correct in the while
	if(intraVLCFormat == 0 && intra == 1)
	{
		bstream.table = (VLC_ENTRY *)DCT_CoefVLC0 + 1;
		bstream.indices = (unsigned char *) DCT_CoefIDX0m;
	}

	int eob_not_read = 1;
	while(eob_not_read)
	{
		// read VLC
		val = bstream.ReadVLC();
		_ASSERTX(val >= 0 && val < 128);

		if(!(intraVLCFormat && intra) && n == 0)
		{
			bstream.table = (VLC_ENTRY *)DCT_CoefVLC0 + 1;
			bstream.indices = (unsigned char *) DCT_CoefIDX0m;
		}

		// lookup entry
		if(intraVLCFormat && intra)	{ pEntry = (BLOCK_ENTRY *) &DCT_Coef1[val];		bitSize += DCT_CoefVLC1[val].zeroes     + DCT_CoefVLC1[val].digits; }
		else if(n != 0)				{ pEntry = (BLOCK_ENTRY *) &DCT_Coef0[val + 1]; bitSize += DCT_CoefVLC0[val + 1].zeroes + DCT_CoefVLC0[val + 1].digits; }
		else						{ pEntry = (BLOCK_ENTRY *) &DCT_Coef0[val];		bitSize += DCT_CoefVLC0[val].zeroes     + DCT_CoefVLC0[val].digits; }

		// set run+level;
		run = pEntry->run;
		level = pEntry->level;

		// read sign
		if(pEntry->run >= 0)			{ negative = bstream.ReadBit(); bitSize++; }

		// if escape encountered then read fixed bit run+level
		if(pEntry->run == -2)
		{
			run = bstream.ReadBits(6);
			level = bstream.ReadBits(12);
			// value 0 for level is forbidden and 2048 not defined
			_ASSERTX(level != 0);
			_ASSERTX(level != 2048);
			// offset level to right sign;
			if(level > 2047) level -= 2 * 2048;
			negative = 0;
			bitSize += 18;
		}

		// if end of block determined
		if(pEntry->run == -1)
		{
			blockEOF = n;
			eob_not_read = 0;
		}
		// else fill coef matrix with coef
		else
		{
			// set run
//			rCoef[n] = run;
			bl_runBuffer[n] = run;
			// set level
//			Coef[n] = negative ? -level : level;
			bl_coefBuffer[n] = negative ? -level : level;

			// half dequantize
			int sign = 0;
			if(intra == 0) 
			{
//				if(Coef[n] > 0) sign = 1;
//				else if(Coef[n] < 0) sign = -1;
				if(bl_coefBuffer[n] > 0) sign = 1;
				else if(bl_coefBuffer[n] < 0) sign = -1;

			}
//			Coef[n] = ((Coef[n] * 2) + sign) * quantizer_scale;
			bl_coefBuffer[n] = ((bl_coefBuffer[n] * 2) + sign) * quantizer_scale;

			n++;
		}
		_ASSERTX(n <= 64);
	}

	// demo block painting
	if(demoMode)
	{
		if(intra)
		{
			if(rand() % 3000 == 0)
			{
//				if(blockEOF > 1)
				{
					for(int i = 0; i < blockEOF; i++)
					{
//						Coef[i] = rand() % 2;
						bl_coefBuffer[i] = rand() % 2;
					}
				}
			}
		}
	}

	// assume all coeffs were intialized
	// _ASSERTX(n == 64);

	return bitSize;
}
