/*****************************************************************************
 * CompressionStrategy.h: The Miracle behind ShrinkTo5
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

 #pragma once

#include "ISO_DEFS.h"

// definitions for the startcode type 1..3 = I_PICTURE..B_PICTURE
#define SEQUENCE  -2
#define EXTENSION -1
#define GOP		  0 
#define SLICE	  4 

typedef struct tagSTARTCODE_POS_ENTRY
{
	signed char entryType;
	int pos;
	unsigned char skip;
} STARTCODE_POS_ENTRY;

typedef struct tagSTARTCODE_POS
{
	int maxNumEntries;
	int numEntries;
	STARTCODE_POS_ENTRY *pEntries;
} STARTCODE_POS;

typedef struct tagPICTURE_SIZE
{
	int pictureType;
	int size;
	int sliceIdx;
	int numSlices;
} PICTURE_SIZE;

typedef struct tagSLICE_SIZE
{
	int size;
	int refIdx;
} SLICE_SIZE;

class CompressionStrategy
{
public:
	CompressionStrategy(void);
	virtual ~CompressionStrategy(void);

	virtual void CreateStrategy(STARTCODE_POS *posTable, int seqSize, int numIPictures, int numPPictures, int numBPictures, int skipAmount);

protected:
	STARTCODE_POS *posTable;
	int skipAmount, seqSize, sumSliceSize;
	int numIPictures, numPPictures, numBPictures, numPictures;
	int sizeIPictures, sizePPictures, sizeBPictures;
	int effSeqSize;

	PICTURE_SIZE *picSize;
	SLICE_SIZE   *sliSize;

	int maxNumPics, maxNumSlices;
	int currNumPics, currNumSlices;

	virtual void AssignLengths();
	virtual void ReAllocMemory();
	virtual void MakeStrategy();
	virtual void CalcSkips(int pictureType, int sizePictures, int takeAmount);
};
