/*****************************************************************************
 * IfoEdit_def.h: Definitions for IFOEdit Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#pragma once
#define WIN32_LEAN_AND_MEAN
#include <vector>
#include <windows.h>

#define SECTOR_SIZE 2048

// Sectors
#define VTSI_MAT		0

typedef struct {
	BOOL isBitrateCBR;
	int nbSubtitles;
	int iSubtitleIDs[32];
	char strSubtitleLanguage[32][2];
	int nbAudioStreams;
	int iAudioIDs[8];
	char strAudioLanguage[8][2];
	int iAudioSampelRates[8];
	int iAudioCodingModes[8];
	int iAudioChannels[8];
} IFO_EDIT_PARAMS;


// Video Attributes
typedef struct
{
	unsigned char auto_letterbox	:1;
	unsigned char auto_Pan_Scan		:1;
	unsigned char aspect			:2;
	unsigned char tv_system			:2;
	unsigned char coding_mode       :2;
	unsigned char film_mode			:1;
	unsigned char letterboxed		:1;
	unsigned char source_resolution :2;
	unsigned char bit_rate			:1;
	unsigned char 					:1;
	unsigned char CC_line_21_1		:1;
	unsigned char CC_line_21_2		:1;
} VTSI_MAT_VIDEO_ATTRIBUTES;

// Audio Attributes
typedef struct
{
	unsigned char application_mode	:2;
	unsigned char language_type		:2;
	unsigned char multichannel_ext  :1;
	unsigned char coding_mode		:3;

	unsigned char channels			:3;
	unsigned char					:1;
	unsigned char sample_rate		:2;
	unsigned char Quantization_DRC	:2;

	unsigned char lang_code_1		:8;
	unsigned char lang_code_2		:8;
	unsigned char language_ext		:8;
	unsigned char code_ext			:8;
	unsigned char					:8;
	unsigned char app_info			:8;
} VTSI_MAT_AUDIO_ATTRIBUTES;

// Subpicture Attributes
typedef struct
{
	unsigned char language_type		:2;
	unsigned char					:3;
	unsigned char coding_mode		:3;

	unsigned char					:8;
	unsigned char lang_code_1		:8;
	unsigned char lang_code_2		:8;
	unsigned char language_ext		:8;
	unsigned char code_ext			:8;
} VTSI_MAT_SUBT_ATTRIBUTES;

typedef struct {
	unsigned char VTS_indetifier[12];
	unsigned int last_VTS_sector;
	unsigned int last_VTSI_sector;
	unsigned char version_number[2];
	unsigned int VTS_category;
	unsigned int end_of_VTSI_MAT;

	unsigned int offset_VTSM_VOBS;		// start sector of Menu Vob
	unsigned int offset_VTSTT_VOBS;		// start sector of Title Vob
	unsigned int offset_VTS_PTT_SRPT;	// (table of Titles and Chapters)
	unsigned int offset_VTS_PGCIT;		// (Title Program Chain table)
	unsigned int offset_VTSM_PGCI_UT;	// (Menu Program Chain table)
	unsigned int offset_VTS_TMAPT;		// (time map)
	unsigned int offset_VTSM_C_ADT;		// (menu cell address table)
	unsigned int offset_VTSM_VOBU_ADMAP; // (menu VOBU address map)
	unsigned int offset_VTS_C_ADT;		// (title set cell address table)
	unsigned int offset_VTS_VOBU_ADMAP;	// (title set VOBU address map)

	// VTSM attributes
	VTSI_MAT_VIDEO_ATTRIBUTES VTSM_video_attribute;
	unsigned short nb_VTSM_audio_attributes;		// Number of audio streams.
	VTSI_MAT_AUDIO_ATTRIBUTES VTSM_audio_attributes[8];
	unsigned short nb_VTSM_subt_attribute;		// Number of subtitle streams.
	VTSI_MAT_SUBT_ATTRIBUTES VTSM_subtitle_attribute;

	// VTSTT attributes
	VTSI_MAT_VIDEO_ATTRIBUTES VTSTT_video_attribute;
	unsigned short nb_VTSTT_audio_attributes;		// Number of audio streams.
	VTSI_MAT_AUDIO_ATTRIBUTES VTSTT_audio_attributes[8];
	unsigned short nb_VTSTT_subt_attribute;		// Number of subtitle streams.
	VTSI_MAT_SUBT_ATTRIBUTES VTSTT_subtitle_attributes[32];
} VTSI_MAT_DATA;

// VTS_PGC
typedef struct {
	char number		:3;
	char			:4;
	char available  :1;
	char reserved   :8;
} PGC_AST_CTL;

typedef struct {
	char number_4_3		:5;
	char				:2;
	char available		:1;
	char number_wide	:5;
	char				:3;
	char number_letterbox :5;
	char				:3;
	char number_pan_scan :5;
	char				:3;
} PGC_SPST_CTL;

typedef struct {
	unsigned char c[8];
} CHAR_8;

typedef struct {
	unsigned short nbPreCommands;
	unsigned short nbPostCommands;
	unsigned short nbCellCommands;
	unsigned short end_address;

	std::vector<CHAR_8> pre_commands;
	std::vector<CHAR_8> post_commands;
	std::vector<CHAR_8> cell_commands;

} PGC_COMMAND_TABLE;

typedef struct {
	std::vector<unsigned char> entry_cell_number;
} PGC_PROGRAM_MAP;

typedef struct {
	unsigned char angle;
	unsigned char SCR_discontinuity;
	unsigned char interleaved;
	unsigned char seamless_playback;
	unsigned char block_type;
	unsigned char cell_type;
	unsigned char restricted;
	unsigned char cell_still_time;
	unsigned char cell_command;
	int cell_playback_time;
	int first_VOBU_start;
	int first_ILVU_end;
	int last_VOBU_start;
	int last_VOBU_end;
} PGC_CELL_PLAYBACK_INFO;

typedef struct {
	unsigned short vob_id;
	unsigned char reserved;
	unsigned char cell_id;
} PGC_CELL_POSITION_INFO;

typedef struct {
	char nb_programs;
	char nb_cells;
	int playback_time;
	int prohibited_ops;

	PGC_AST_CTL audio[8];
	PGC_SPST_CTL subpicture[32];

	short next_PGCN;
	short prev_PGCN;
	short goup_PGCN;
	char PGC_still_time;
	char playback_mode;
	__int32 palette[16];
	short offset_commands;
	short offset_program_map;
	short offset_playback_info;
	short offset_position_info;

	PGC_COMMAND_TABLE command_table;
	PGC_PROGRAM_MAP program_map;
	std::vector<PGC_CELL_PLAYBACK_INFO> cell_playback_info;
	std::vector<PGC_CELL_POSITION_INFO> cell_position_infos;

} VTS_PGC;

// VTS_PGCITI
typedef struct {
	short nb_PGCs;
	int VTS_PGCI_SRP_end;
	int PGC_category;
	int offset_VTS_PGC;
	std::vector<VTS_PGC> PGCs;
} VTS_PGCITI;


// VTS_PTT_SRPT
typedef struct {
	short PGCN;
	short PGN;
}VTS_PTT;

typedef struct {
	short nbTitles;
	int end_address;
	int offset_PTT;
	std::vector<VTS_PTT> vts_ptt;
}VTS_PTT_SRPT;

// VMGM_PGCI_UT
typedef struct {
	char category[4];
	unsigned __int32 offset_PGCI;
}PGC_MENU;

// VTSM_PGCI_UT
typedef struct {
	unsigned short nbMenus;
	unsigned int end_adress;
	std::vector<PGC_MENU> Menus;
	std::vector<VTS_PGC> vtsm_pgc;
}VTSM_LU;

typedef struct {
	unsigned char lang_code[2];
	unsigned char lang_code_ext;
	unsigned char existence_flags;
	unsigned int offset_VTSM_LU;
}VTSM_LU_INFO;

typedef struct {
	unsigned short nbUnits;
	unsigned int end_adress;
	std::vector<VTSM_LU_INFO> vtsm_lu_infos;
	std::vector<VTSM_LU> vtsm_lu;
}VTSM_PGCI_UT;

// VTS_TMAPTI
typedef struct {
	unsigned char time_unit;
	unsigned short nbEntries;
	std::vector<__int32> entries;
} VTS_TMAP;

typedef struct {
	short nbPGCs;
	int end_adress;
	int offset_VTS_TMAP;
	VTS_TMAP vts_map;
} VTS_TMAPTI;

// VTS_C_ADT
typedef struct {
	unsigned short vobID;
	unsigned char cellID;
	unsigned int start_sec_VOB;
	unsigned int end_sec_VOB;
} VTS_C_ADT_ENTRY;

typedef struct {
	unsigned short nbCells;
	unsigned int end_adress;
	std::vector<VTS_C_ADT_ENTRY> entries;
} VTS_C_ADT;

// VTS_VOBU_ADMAP
typedef struct {
	int end_adress;
	int star_sec_first_VOBU;
} VTS_VOBU_ADMAP;

// VMG_MAT
typedef struct {
	unsigned char VMG_indetifier[12];
	unsigned int last_VMG_sector;
	unsigned int last_IFO_sector;
	unsigned char version_number[2];
	unsigned int VMG_category;
	unsigned short nbVolumes;
	unsigned short vol_number;
	unsigned char side_ID;
	unsigned short nb_title_sets;
	unsigned char provider_ID[32];
	unsigned char VMG_POS[8];
	unsigned int end_of_VMGI_MAT;

	unsigned int offset_FP_PGC;			// First Play program chain
	unsigned int offset_VMGM_VOBS;		// start sector of Menu Vob
	unsigned int offset_TT_SRPT;		// table of titles
	unsigned int offset_VMGM_PGCI_UT;   // Menu Program Chain table
	unsigned int offset_VMG_PTL_MAIT;   // Parental Management masks
	unsigned int offset_VMG_VTS_ATRT;	// copies of VTS audio/sub-picture attributes
	unsigned int offset_VMG_TXTDT_MG;   // text data
	unsigned int offset_VMGM_C_ADT;		// menu cell address table
	unsigned int offset_VMGM_VOBU_ADMAP;// menu VOBU address map

	// VMGM_VOBS attributes
	VTSI_MAT_VIDEO_ATTRIBUTES VMGM_video_attribute;
	unsigned short nb_VMGM_audio_attributes;		// Number of audio streams.
	VTSI_MAT_AUDIO_ATTRIBUTES VMGM_audio_attributes[8];
	unsigned short nb_VMGM_subt_attribute;		// Number of subtitle streams.
	VTSI_MAT_SUBT_ATTRIBUTES VMGM_subtitle_attribute;

	VTS_PGC FP_PGC; // First Play program chain
} VMG_MAT_DATA;

// VMG_TT_SRPT
typedef struct {
	unsigned char title_type;
	unsigned char nbAngels;
	unsigned short nbChapters;
	unsigned short par_mng_mask;
	unsigned char VTSN;
	unsigned char VTS_TTN;
	unsigned int start_sec_VTS;
}VMG_TT_SRPT_ENTRY;

typedef struct {
	unsigned short nbTitles;
	unsigned int end_adress;
	std::vector<VMG_TT_SRPT_ENTRY> entries;
}VMG_TT_SRPT;

// VMGM_PGCI_UT

typedef struct {
	unsigned short nbMenus;
	unsigned int end_adress;
	std::vector<PGC_MENU> Menus;
	std::vector<VTS_PGC> PGCs;
} VMGM_LU;

typedef struct {
	unsigned short nbLangUnits;
	unsigned int end_adress;
	unsigned char lang_code[2];
	unsigned char lang_ext;
	unsigned char menu_existence_flag;
	unsigned int offset_VMGM_LU;
	std::vector<VMGM_LU> VMGM_LUs;
} VMGM_PGCI_UT;

// VMG_PTL_MAIT
typedef struct {
	unsigned short country_code;
	unsigned short offset_PTL_MAIT;
} VMG_PTL_MAIT_CONTRY_CODE;
typedef struct {
	std::vector<unsigned short> par_masks;
} VMG_PTL_MAIT_MASK;

typedef struct {
	unsigned short nbCountries;
	unsigned short nbTitleSets;
	unsigned int end_PTL_MAIT;
	std::vector<VMG_PTL_MAIT_CONTRY_CODE> country_codes;
	VMG_PTL_MAIT_MASK PTL_MAITs[8];
} VMG_PTL_MAIT;

// VMG_VTS_ATRT
typedef struct {
	unsigned int end_address;
	unsigned int VTS_CAT;
	// VTSM attributes
	VTSI_MAT_VIDEO_ATTRIBUTES VTSM_video_attribute;
	unsigned short nb_VTSM_audio_attributes;		// Number of audio streams.
	VTSI_MAT_AUDIO_ATTRIBUTES VTSM_audio_attributes[8];
	unsigned short nb_VTSM_subt_attribute;		// Number of subtitle streams.
	VTSI_MAT_SUBT_ATTRIBUTES VTSM_subtitle_attribute;

	// VTSTT attributes
	VTSI_MAT_VIDEO_ATTRIBUTES VTSTT_video_attribute;
	unsigned short nb_VTSTT_audio_attributes;		// Number of audio streams.
	VTSI_MAT_AUDIO_ATTRIBUTES VTSTT_audio_attributes[8];
	unsigned short nb_VTSTT_subt_attribute;		// Number of subtitle streams.
	VTSI_MAT_SUBT_ATTRIBUTES VTSTT_subtitle_attributes[32];
} VTS_ATRT;

typedef struct {
	unsigned short nbTitleSets;
	unsigned int VTS_ATRT_end;
	std::vector<int> VTS_ATRT_offsets;
	std::vector<VTS_ATRT> VTS_ATRTs;
} VMG_VTS_ATRT;

#define VMGM_C_ADT VTS_C_ADT
#define VMGM_VOBU_ADMAP VTS_VOBU_ADMAP

/***********************************************/