/*****************************************************************************
 * IFOFile.cpp: Class to read and write from/to IFO FIles
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include "StdAfx.h"
#include "ifofile.h"
#include "malloc.h"

CIFOFile::CIFOFile(void)
{
	m_buffer = NULL;
}

CIFOFile::CIFOFile(const char *strIfoSrcName)
{
	m_buffer = NULL;
	OpenFile(strIfoSrcName);
}

CIFOFile::~CIFOFile(void)
{
	CloseFile();
}

int CIFOFile::OpenFile(const char *strIfoSrcName)
{
	HANDLE hFile;
	hFile = CreateFile(strIfoSrcName,
					   GENERIC_READ,
					   0,0,
					   OPEN_EXISTING,
					   FILE_ATTRIBUTE_NORMAL,
					   0);
	if(hFile == INVALID_HANDLE_VALUE) return 0;

	GetFileSizeEx(hFile,  &m_iSize);

	m_buffer = (unsigned char*)malloc(m_iSize.QuadPart);
	if(m_buffer == NULL)
	{
		CloseHandle(hFile);
		return 0;
	}

	DWORD dw;
	ReadFile(hFile,m_buffer,m_iSize.QuadPart,&dw,NULL);
	CloseHandle(hFile);
	if(dw != m_iSize.QuadPart) return 0;
	return 1;
}

int CIFOFile::SaveFile(const char *strIfoDestName)
{
	HANDLE hFile;
	hFile = CreateFile(strIfoDestName,
					   GENERIC_WRITE,
					   0,0,
					   CREATE_ALWAYS,
					   FILE_ATTRIBUTE_NORMAL,
					   0);
	if(hFile == INVALID_HANDLE_VALUE) return 0;

	DWORD dw;
	WriteFile(hFile,m_buffer,m_iSize.QuadPart,&dw,NULL);
	CloseHandle(hFile);
	if(dw != m_iSize.QuadPart) return 0;
	return 1;
}

void CIFOFile::CloseFile(void)
{
	free(m_buffer);
	m_buffer = NULL;
}

__int64 CIFOFile::PutINT64(int offset, unsigned __int64 value)
{
	unsigned __int64 i=0;
	i = ((value<<48)&0xFF0000000000) | ((value<<24)&0xFF00000000) |
		((value<<8)&0xFF000000) | ((value>>8)&0xFF0000) |
		((value>>24)&0xFF00) | ((value>>48)&0xFF);
	memcpy(m_buffer+offset,&i,sizeof(__int64));

	memcpy(m_buffer+offset,&value,sizeof(value));
	return value;
}
int CIFOFile::PutINT32(int offset, unsigned int value)
{
	unsigned __int32 i=0;
	i = ((value<<24)&0xFF000000) | ((value<<8)&0xFF0000) |
		((value>>8)&0xFF00) | ((value>>24)&0xFF);
	memcpy(m_buffer+offset,&i,sizeof(__int32));
	return value;
}
unsigned char CIFOFile::PutBIT(int offset_bytes, int offset_bits, unsigned char value)
{
	unsigned char c,tmp;
	memcpy(&tmp,m_buffer+offset_bytes,sizeof(tmp));
	c=0x01;
	c<<=offset_bits;
	c ^= 0xFF;
	tmp &= c;
	tmp |= ((value<<offset_bits)&c);
	return value;
}
unsigned char CIFOFile::PutBYTE(int offset, unsigned char value)
{
	memcpy(m_buffer+offset,&value,sizeof(value));
	return value;
}
unsigned short CIFOFile::PutSHORT(int offset, unsigned short value)
{
	unsigned short s=0;
	s = ((value<<8)&0xFF00) |((value>>8)&0xFF);
	memcpy(m_buffer+offset,&s,sizeof(s));
	return value;
}
void CIFOFile::PutSTRING(int offset, const char* value)
{
	int len = strlen(value);
	memcpy(m_buffer+offset,value,len);
}

void CIFOFile::PutMEM(int offset,int len, unsigned char* bytes)
{
	memcpy(m_buffer+offset,bytes,len);
}

void CIFOFile::GetMEM(int offset,int len, BYTE *bytes)
{
	memcpy(bytes,m_buffer+offset,len);
}

void CIFOFile::GetSTRING(int offset,int len, char* str)
{
	memcpy(str,m_buffer+offset,len);
	str[len] = '\0';
}
unsigned short CIFOFile::GetSHORT(int offset)
{
	unsigned short s=0;
	s = (*(m_buffer+offset))<<8 | (*(m_buffer+offset+1));
	return s;
}
unsigned char CIFOFile::GetBYTE(int offset)
{
	return (unsigned char)*(m_buffer+offset);
}
char CIFOFile::GetBIT(int byte_offset, int bit_offset)
{
	unsigned char c = *(m_buffer+byte_offset);
	c>>=bit_offset;
	return c&0x01;
}

char CIFOFile::GetBITS(int byte_offset, int bit_offset, int n)
{
	char ret=0x00;
	for(int i=0;i<n;i++)
	{
		char c = GetBIT(byte_offset,bit_offset+i);
		c<<=i;ret|=c;
	}
	return ret;
}

unsigned int CIFOFile::GetINT32(int offset)
{
	unsigned __int32 i=0;
	i = ((*(m_buffer+offset))<<24) | ((*(m_buffer+offset+1))<<16) |
		((*(m_buffer+offset+2))<<8) | ((*(m_buffer+offset+3)));
	return i;
}
unsigned __int64 CIFOFile::GetINT64(int offset)
{
	unsigned __int64 i=0;
	i = ((*(m_buffer+offset))<<40) | ((*(m_buffer+offset+1))<<32) |
		((*(m_buffer+offset+2))<<24) | ((*(m_buffer+offset+3))<<16) |
		((*(m_buffer+offset+4))<<8) | ((*(m_buffer+offset+5)));
	return i;
}


