/*****************************************************************************
 * IFOFile.h: Class to read and write from/to IFO FIles
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once
//#define WIN32_LEAN_AND_MEAN
#include <windows.h>

class CIFOFile
{
	LARGE_INTEGER m_iSize;
	unsigned char* m_buffer;

protected:
	CIFOFile(void);
	CIFOFile(const char *strIfoSrcName);
	virtual ~CIFOFile(void);
	int OpenFile(const char *strIfoSrcName);
	int SaveFile(const char *strIfoDestName);
	void CloseFile(void);

	// save methods
	__int64 PutINT64(int offset, unsigned __int64 value);
	int PutINT32(int offset, unsigned int value);
	unsigned char PutBIT(int offset_bytes, int offset_bits, unsigned char value);
	unsigned char PutBYTE(int offset, unsigned char value);
	unsigned short PutSHORT(int offset, unsigned short value);
	void PutSTRING(int offset, const char* value);
	void PutMEM(int offset,int len, unsigned char* bytes);

	// load methods
	void GetMEM(int offset,int len, unsigned char* bytes);
	void GetSTRING(int offset,int len, char* str);
	unsigned short GetSHORT(int offset);
	unsigned char GetBYTE(int offset);
	char GetBIT(int byte_offset, int bit_offset);
	char GetBITS(int byte_offset, int bit_offset, int n);
	unsigned int GetINT32(int offset);
	unsigned __int64 GetINT64(int offset);

	// load/save
	unsigned char TransferBIT(int offset_byte,int offset_bit, bool save, unsigned char value=0) {
		return save ? PutBIT(offset_byte,offset_bit,value) : GetBIT(offset_byte,offset_bit);};
	unsigned char TransferBYTE(int offset, bool save, unsigned char value=0) {
		return save ? PutBYTE(offset,value) : GetBYTE(offset); };
	unsigned short TransferSHORT(int offset, bool save, unsigned short value=0) {
		return save ? PutSHORT(offset,value) : GetSHORT(offset); };
	unsigned int TransferINT32(int offset, bool save, unsigned int value=0) {
		return save ? PutINT32(offset,value) : GetINT32(offset); };
	unsigned __int64 TransferINT64(int offset, bool save, unsigned __int64 value=0) {
		return save ? PutINT64(offset,value) : GetINT64(offset); };
	void TransferMEM(int offset, int len, BYTE *data, bool save){
		save ? PutMEM(offset,len,data) : GetMEM(offset,len,data);};

};

