/*****************************************************************************
 * Multiplexer.cpp: Multiplexer Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "transfer.h"
#include "multiplexer.h"
#include "..\resource.h"

Multiplexer::Multiplexer(void)
{
	int i;

	// set constants
	SECTOR_SIZE = 2048;
	MAX_SECTORS = 10240;
//	MAX_SECTORS = 2048;

	// allocate memory
	pSeqBuffer  = new unsigned char[SECTOR_SIZE * MAX_SECTORS];
	pData       = new unsigned char[SECTOR_SIZE * MAX_SECTORS];
	pPackBuffer = (PESPack **) new int[MAX_SECTORS];
	pPackBufferFinalized = (PESPack **) new int[MAX_SECTORS];
	for(i = 0; i < MAX_SECTORS; i++)
	{
		pPackBuffer[i] = new PESPack();
		pPackBufferFinalized[i] = new PESPack();
	}

	IgnoreTable = NULL;
	numPicInfo = 0;

	alert = NULL;
	hCSS = NULL;

	demoMode = 0;
	removePUOs = 0;
	quality = 1;

	// load hyperspeed bitmap
	hHyperSpeedBMP = LoadImage(NULL, "hyperspeed.bmp", IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	if(hHyperSpeedBMP == NULL) hHyperSpeedBMP = LoadBitmap(GetModuleHandle("ShrinkTo5.dll"), MAKEINTRESOURCE(IDB_HYPERSPEED));

	VideoSeq.hHyperSpeedBMP = hHyperSpeedBMP;
}

Multiplexer::~Multiplexer(void)
{
	int i;

	// free memory
	delete pSeqBuffer;
	delete pData;
	for(i = 0; i < MAX_SECTORS; i++)
	{
		delete pPackBuffer[i];
		delete pPackBufferFinalized[i];
	}
	delete pPackBuffer;
	delete pPackBufferFinalized;

	if(hCSS != NULL)
	{
		CSS->CSSDestroy(hCSS);
		hCSS = NULL;
	}

	// close IO layers
	ReadIO.Close();
	WriteIO.Close();

	// cleanup hyperspeed bitmap
	DeleteObject(hHyperSpeedBMP);
}

int Multiplexer::Init(char *srcPath, char *targetPath, int VTSnum, int VOBnum, IFO *pIFO, int tsSrcReadSECT, int tsTargetWrittenSECT)
{
	// initialize parameters
	this->srcPath		= srcPath;
	this->targetPath	= targetPath;
	this->VTSnum		= VTSnum;
	this->VOBnum		= VOBnum;
	this->pIFO			= pIFO;
	this->tsSrcReadSECT = tsSrcReadSECT;
	this->tsTargetWrittenSECT	 = tsTargetWrittenSECT;

	tsSrcReadSECTVOB	   = 0;
	tsTargetWrittenSECTVOB = 0;

	CSS = NULL;

	// open IO layers (source and target file)
	char actualSrcPath[256];
	if(VTSnum == 0)	sprintf(actualSrcPath, "%sVIDEO_TS.VOB", srcPath);
	else			sprintf(actualSrcPath, "%sVTS_%02d_%d.VOB", srcPath, VTSnum, VOBnum);
	sprintf(actualFileName, "%s", actualSrcPath);

	char actualTargetPath[256];
	if(VTSnum == 0)	sprintf(actualTargetPath, "%sVIDEO_TS.VOB", targetPath);
	else			sprintf(actualTargetPath, "%sVTS_%02d_%d.VOB", targetPath, VTSnum, VOBnum);

	if(alert != NULL) alert->SendAlert(ALERT_FILENAME_CHANGED, (int)actualSrcPath);

//	ReadIO.Close();
//	WriteIO.Close();

	ReadIO.Open((unsigned char *)actualSrcPath, 0);
	WriteIO.Open((unsigned char *)actualTargetPath, 1);

	overShootNumPicInfo = 0;
	numPicInfo = 0;
	savedByte[0] = 0xFF;
	savedByte[1] = 0xFF;
	savedByte[2] = 0xFF;
	savedPackIndex = -1;

	packCount = 0;
	packetsWriteWait = 0;
	overShootSize = 0;
	pictureCount = 0;
	ignoreCounter = 0;
	VideoSeq.BitRateControl.inputBytes = 0;
	VideoSeq.BitRateControl.outputBytes = 0;
	VideoSeq.BitRateControl.inputBytesSeq = 0;
	VideoSeq.BitRateControl.outputBytesSeq = 0;

	// get next sector entry from IFO to transform
	currTransformSECTVOB = pIFO->GetNextSectorEntry(VOBnum == 0 ? REFERENCE_MENU_VOB : REFERENCE_TITLE_VOB);
	currTransformSECTIFO = pIFO->GetNextSectorEntry(REFERENCE_IFO);
	_ASSERT(currTransformSECTIFO >= tsSrcReadSECT - 1);
	_ASSERT(currTransformSECTVOB >= tsSrcReadSECTVOB - 1);

	srand(0); // for the slice skipping code in picture.cpp

	return 0;
}

void Multiplexer::Close()
{
	// close IO layers
	ReadIO.Close();
	WriteIO.Close();
}

int Multiplexer::EnableCSS(int enable)
{
	if(enable) 
	{
		// if we have no CSS library available return
		if(CSS->isAvailable == 0) return -1;

		// init CSS
		// if decryption fails, then try some panic solutions
		if(InitCSS(VTSnum, VOBnum) != 0) 
		{
			int panicVTSnum = VTSnum;
			int panicVOBnum = VOBnum;

			panicVOBnum = 0;

			// go trough all titlesets till a key is found
			int retval = 0;
			int wraparound = 0;
			int menupresent = 0;
			do
			{
				retval = InitCSS(panicVTSnum, panicVOBnum);

				// see whether menu is present
				if(panicVOBnum == 0 && retval != -1) menupresent = 1;
				if(panicVOBnum == 0 && retval == -1) menupresent = 0;

				// if we have come at the end of the DVD then search at the beginning
				if(retval == -1 && panicVOBnum == 1 && menupresent == 0 && wraparound == 0) { panicVTSnum = 0; panicVOBnum = 0; wraparound = 1; };
				if(retval == -1 && panicVOBnum == 1 && menupresent == 0 && wraparound == 1) break;

				// if we are at the VIDEO_TS
				if(retval != 0  && panicVTSnum == 0) { panicVTSnum = 1; panicVOBnum = 0; };
				// if we are at the end of a VTS
				if(retval == -1 && panicVOBnum > 0) { panicVTSnum++; panicVOBnum = 0; }
				// if we are at the beginning/in the middle of a VTS
				if(retval == -2) panicVOBnum++;
			} while(retval != 0);
			if(retval != 0) return -2;
		}
	}
	else
	{
		// delete the handle
		if(hCSS != NULL) 
		{
			CSS->CSSDestroy(hCSS);
			hCSS = NULL;
		}
	}


	// set CSS (if NULL stream will not be decrypted)
	for(int i = 0; i < MAX_SECTORS; i++)
	{
		pPackBuffer[i]->CSS = CSS;
		pPackBuffer[i]->hCSS = hCSS;
		pPackBufferFinalized[i]->CSS = CSS;
		pPackBufferFinalized[i]->hCSS = hCSS;
	}

	return 0;
}

int Multiplexer::InitCSS(int VTSnum, int VOBnum)
{
	char actualPath[256];
	if(VTSnum == 0)	sprintf(actualPath, "%sVIDEO_TS.VOB", srcPath);
	else			sprintf(actualPath, "%sVTS_%02d_%d.VOB", srcPath, VTSnum, VOBnum);

	// check whether file exists
	// if not, return -1
	WIN32_FILE_ATTRIBUTE_DATA fileData;
	if(GetFileAttributesEx(actualPath, GetFileExInfoStandard, &fileData) == 0) return -1;

	// *** init CSS
	hCSS = CSS->CSSCreate();
	int retval = CSS->CSSOpen(actualPath, hCSS);
	CSS->CSSClose(hCSS);

	// if decryption fails
	if(retval != 0) 
	{
		CSS->CSSDestroy(hCSS);
		hCSS = NULL;
		return -2;
	}

	return 0;
}

void Multiplexer::SetCSS(Machinist *CSS)
{
	this->CSS = CSS;
}

void Multiplexer::SetRemovePUOs(bool removePUOs)
{
	this->removePUOs = removePUOs;
}

int Multiplexer::GetRemovePUOs()
{
	return removePUOs;
}

void Multiplexer::SetDemoMode(bool demoMode)
{
	this->demoMode = demoMode;
}

int  Multiplexer::GetDemoMode()
{
	return demoMode;
}

// reads a sequence and parses it
int Multiplexer::ReadSequence()
{
	int ret;

	// initialize parameters
	seqSize = 0;

/*c	// if a block was read too much then use it now
	if(overShoot)
	{
		// move overshoot packet at the beginning
		PESPack *tPack = pPackBuffer[0];
		pPackBuffer[0] = pPackBuffer[overShoot];
		pPackBuffer[overShoot] = tPack;

		// copy pack payload to sequence buffer
		int tSize = pPackBuffer[0]->Packet.payLoadSize;
		CopyMemory(pSeqBuffer, pPackBuffer[0]->Packet.payLoad, tSize);
		seqSize += tSize;
		packCount++;
	}
	overShoot = 0;*/

	// read (rest of) next sequence
	while((ret = ReadNextPack()) == 1);

	// if packets are scrambled
	if(ret == -2) return -2;

	// if no next sequence pack is available
	if(seqSize == 0 && ret == 2) return 2;	// if there is another NAVPack
	else if(seqSize == 0) return -1;		// if there is no other NAVPack

	// parse sequence at higher levels
	pSeqBuffer[seqSize]     = 0;
	pSeqBuffer[seqSize + 1] = 0;
	pSeqBuffer[seqSize + 2] = 0;
	VideoSeq.absNumPicture = pictureCount;
	VideoSeq.demoMode = demoMode;

	// get compression ratio
	float ratio = VideoSeq.BitRateControl.RecalcRatio();
	dataSize = -1;
	// if ratio suggest more compression then recode
	if(ratio < 1)
	{
		try
		{
			dataSize = VideoSeq.AttachBuffer(pSeqBuffer, seqSize, pData);
		}
		catch(...)
		{
			VideoSeq.skipped = 1;		
		}
	}
	// else skip recoding
	else
	{ 
		// equivalent to skipped = 1 but gives nice hyperspeed logo
		VideoSeq.SetHyperSpeed();
	}

	// set bitrate control input statistics 
	// must be made after sequence processing to have a consistent input/output ratio measurement
	VideoSeq.BitRateControl.inputBytes += packCount * pPackBuffer[0]->SECTOR_SIZE;
	VideoSeq.BitRateControl.inputBytesSeq = packCount * pPackBuffer[0]->SECTOR_SIZE;

	// count statistics
	compressionRatio = 1 / VideoSeq.BitRateControl.GetCurrRatio();
	pictureCount += VideoSeq.numPictures;

	// if next NAVPack was found
	if(ret == 2) return 2;

	return 1;
}	

// reads a pack and categorize it
int Multiplexer::ReadNextPack()
{
	int ret;

	// see if we have an overshoot
	if(overShootSize > 0)
	{
		// copy overshoot into buffer
		CopyMemory(pSeqBuffer, overShootBuffer, overShootSize);
		seqStreamID = overShootStreamID;
		seqSize = overShootSize;
		overShootSize = 0;
		packCount = 1;

//		ExtractPicInfo(0, 2);
	}

	// read next pack
	ret = pPackBuffer[packCount]->Read(&ReadIO);

	// if end of file reached
	// see if we have another VOB to read
    if(ret == -1)
	{
		// if it is not the video manager or menu vob
		if(VTSnum > 0 && VOBnum > 0)
		{
			VOBnum++;

			// open IO layers (source and target file)
			char actualSrcPath[256];
			if(VTSnum == 0)	sprintf(actualSrcPath, "%sVIDEO_TS.VOB", srcPath);
			else			sprintf(actualSrcPath, "%sVTS_%02d_%d.VOB", srcPath, VTSnum, VOBnum);

			char actualTargetPath[256];
			if(VTSnum == 0)	sprintf(actualTargetPath, "%sVIDEO_TS.VOB", targetPath);
			else			sprintf(actualTargetPath, "%sVTS_%02d_%d.VOB", targetPath, VTSnum, VOBnum);

			// test whether file is available
			WIN32_FILE_ATTRIBUTE_DATA fileData;
			int avail = GetFileAttributesEx(actualSrcPath, GetFileExInfoStandard, &fileData);
			// if it is then open it
			if(avail != 0) 
			{
				ReadIO.Close();
				WriteIO.Close();

				ReadIO.Open((unsigned char *)actualSrcPath, 0);
				WriteIO.Open((unsigned char *)actualTargetPath, 1);

				sprintf(actualFileName, "%s", actualSrcPath);

				ret = pPackBuffer[packCount]->Read(&ReadIO);

				if(ret >= 0 && alert != NULL) alert->SendAlert(ALERT_FILENAME_CHANGED, (int)actualSrcPath);
			}
		}
	}

	// if no error
	if(ret >= 0)
	{
		// initialize packet as normal
		pPackBuffer[packCount]->category = CATEGORY_NORMAL;

		// if pack is a video stream
		if(pPackBuffer[packCount]->Packet.streamID >= 0xE0 && pPackBuffer[packCount]->Packet.streamID <= 0xEF)
		{
			// check for sequence header in the middle of packet
			int posSeq = SearchLastSequenceHeader(pPackBuffer[packCount]);

			// if sequence encountered mark packet as beginning
			if(posSeq >= 0)
			{
				pPackBuffer[packCount]->category = CATEGORY_SEQUENCE_HEADER;

				if(seqSize == 0)
				{
					// copy sequence into buffer
					int tSize = pPackBuffer[packCount]->Packet.payLoadSize;
					CopyMemory(&pSeqBuffer[seqSize], pPackBuffer[packCount]->Packet.payLoad, tSize);
					seqSize += tSize;

					// save the stream id (only one stream at once can be decoded now)
					seqStreamID = pPackBuffer[packCount]->Packet.streamID;
					ret = 1;

					// ExtractPicInfo(packCount, 0);
				}
				else
				{
					// copy remaining sequence into buffer
					int tSize = posSeq;
					CopyMemory(&pSeqBuffer[seqSize], pPackBuffer[packCount]->Packet.payLoad, tSize);
					seqSize += tSize;

					// record overshoot
					tSize = pPackBuffer[packCount]->Packet.payLoadSize - posSeq;
					CopyMemory(overShootBuffer, &pPackBuffer[packCount]->Packet.payLoad[posSeq], tSize);
					overShootSize = tSize;

					// save the stream id in overshoot (only one stream at once can be decoded now)
					overShootStreamID = pPackBuffer[packCount]->Packet.streamID;
					// check current (and overshoot) stream id (only one stream at once can be decoded now)
//					_ASSERT(seqStreamID == pPackBuffer[packCount]->Packet.streamID); 
					ret = 0;

//					ExtractPicInfo(packCount, 1);
				}
			}
			else
			{
				// copy sequence into buffer
				int tSize = pPackBuffer[packCount]->Packet.payLoadSize;
				CopyMemory(&pSeqBuffer[seqSize], pPackBuffer[packCount]->Packet.payLoad, tSize);
				seqSize += tSize;

				// check stream id (only one stream at once can be decoded now)
				// _ASSERT(seqStreamID == pPackBuffer[packCount]->Packet.streamID); 
				ret = 1;

//				ExtractPicInfo(packCount, 0);
			}

			// delete packet payload (charge)
			pPackBuffer[packCount]->Packet.charge_size = 0;
		}
		// if NAVPack found
		else if(pPackBuffer[packCount]->Packet.streamID == 0xBF && packCount != 0)
		{
			// charge not applicable
			pPackBuffer[packCount]->Packet.charge_size = -1;
			ret = 2;
		}
		else
		{
			// charge not applicable
			pPackBuffer[packCount]->Packet.charge_size = -1;
			ret = 1;
		}
	
		// increase read packet counter
		packCount++;
	}

	// assume there is enough storage space
	_ASSERT(packCount < MAX_SECTORS);

	return ret;
}

// very time consuming!
int Multiplexer::SearchLastSequenceHeader(PESPack *pPackBuffer)
{
	PESPacket *Packet = &pPackBuffer->Packet;
	int lastSeq = -1;

	// if pack is a video stream
	if(Packet->streamID >= 0xE0 && Packet->streamID <= 0xEF)
	{
		for(int i = 0; i < Packet->payLoadSize - 3; i++)
		{
			if(Packet->payLoad[i]     == 0x00 &&
			   Packet->payLoad[i + 1] == 0x00 &&
			   Packet->payLoad[i + 2] == 0x01 &&
			   Packet->payLoad[i + 3] == 0xB3)
			{
				lastSeq = i;
			}
		}
	}

	return lastSeq;
}

// write a sequence
int Multiplexer::WriteSequence()
{
	int ret, i;

	// if there is nothing to write
	if(packCount == 0) return 0;

	int overshootNAVPack = 0;

	// retrieve the sequence data
	_ASSERT(dataSize <= seqSize);
	// if data has grown, then skip videoseq recoding
	if(dataSize > seqSize) VideoSeq.skipped = 1;

	// if skipped
	// for stability reasons
	// if data was not shrinked, take original data
	if(VideoSeq.skipped == 1 || seqSize == 0)
	{
		unsigned char *temp = pData;
		pData = pSeqBuffer;
		pSeqBuffer = temp;
		dataSize = seqSize;
	}

//	dataSize = seqSize;
//	ExtractPicInfo(pData, dataSize);

	float contraction = (float) dataSize / (float) seqSize;
	float restPack = 1 - contraction;

	int targetPackPos = 0;
	int dataPos = 0;

	// calculate the contraction ratio
/*	if(dataSize < seqSize)
	{*/
		// copy data into the empty video packets 
		// and move the other packets to their new position in the write stream
		int wasVideoBefore = 0;
		for(i = 0; i < packCount; i++)
		{
			// if it is a video packet
			if(pPackBuffer[i]->Packet.streamID >= 0xE0 && pPackBuffer[i]->Packet.streamID <= 0xEF)
			{
/*				// if packet can be filled and moved
				if(restPack < 1)
				{*/
					// if there is data to fill
//					if(!OmitPack(i))
					{
						if(dataSize != dataPos)
						{
							// move packet at the new position
							PESPack *tPack = pPackBuffer[targetPackPos];
							pPackBuffer[targetPackPos] = pPackBuffer[i];
							pPackBuffer[i] = tPack;

							// calculate new payload size
							int tSize;
							// more than one sequence may be present in one packet
							_ASSERT(pPackBuffer[targetPackPos]->Packet.charge_size < 2048 && pPackBuffer[targetPackPos]->Packet.charge_size >= 0);
							_ASSERT(pPackBuffer[targetPackPos]->Packet.charge_size == 0 || (pPackBuffer[targetPackPos]->category == CATEGORY_SEQUENCE_HEADER && wasVideoBefore == 0));
							int tPos = pPackBuffer[targetPackPos]->Packet.charge_size;
						
							// if first sequence packet then do not load at maximum payload size
							// (hack)
							/*if(dataPos == 0)*/ tSize = pPackBuffer[targetPackPos]->Packet.payLoadSize - tPos;
	//						else			 tSize = pPackBuffer[targetPackPos]->Packet.maxPayLoadSize;
							_ASSERT((tPos == 0 && pPackBuffer[i]->category == CATEGORY_NORMAL) || pPackBuffer[i]->category == CATEGORY_SEQUENCE_HEADER);
							_ASSERT(pPackBuffer[targetPackPos]->Packet.payLoadSize != 0);


							int bytesToCopy = min(tSize, dataSize - dataPos);

							// set new payload size
							// (hack - not for first packet)
							if(bytesToCopy < pPackBuffer[targetPackPos]->Packet.payLoadSize && pPackBuffer[targetPackPos]->category != CATEGORY_SEQUENCE_HEADER) pPackBuffer[targetPackPos]->Packet.SetPayLoadSize(bytesToCopy);
							// copy data into payload
							CopyMemory(&pPackBuffer[targetPackPos]->Packet.payLoad[tPos], &pData[dataPos], bytesToCopy);
							pPackBuffer[targetPackPos]->Packet.charge_size += bytesToCopy;
							dataPos += bytesToCopy;

							// a one pack sequence should not be possible
//							_ASSERT(bytesToCopy != dataSize);

							// set next position in write stream
							targetPackPos++;
						}
					}
/*				}
				// empty packet will be removed from write stream
				else
				{
					restPack -= 1;
				}
				// calculate when next packet will be removed
				restPack += 1 - contraction;*/

				wasVideoBefore = 1;
			}
			else
			{
				int ignore = 0;
				while(tsSrcReadSECTVOB + i >= IgnoreTable[ignoreCounter].nextPGCMark) ignoreCounter++;

				if(IgnoreTable != NULL)
				{
					for(int j = 0; j < IgnoreTable[ignoreCounter].numStream; j++)
					{
						if(IgnoreTable[ignoreCounter].streamID[j] == pPackBuffer[i]->Packet.streamID && IgnoreTable[ignoreCounter].subStreamID[j] == pPackBuffer[i]->Packet.subStreamID)
						{
							ignore = 1;
						}
					}
				}

				// if this is a NAVPack
				if(pPackBuffer[i]->Packet.streamID == 0xBF)
				{
					_ASSERT(ignore == 0);
					pNAV[0].AttachPESPacket(&pPackBuffer[i]->Packet);
					if(removePUOs) pNAV[0].RemoveProhibitedUserOps();
					pNAV[0].SetLogicalBlockNumber(tsTargetWrittenSECTVOB + targetPackPos);
					if(i == packCount - 1) 
					{
						// if this is the first and last GOP
						if(!packetsWriteWait)
						{
							pNAV[0].AttachPESPacket(&pPackBuffer[0]->Packet);
							pNAV[0].SetEndSECT(targetPackPos - 1);
						}
						// if there were other GOPs before
						else
						{
							pNAV[0].AttachPESPacket(&pPackBufferFinalized[0]->Packet);
							pNAV[0].SetEndSECT(packetsWriteWait + targetPackPos - 1);
						}
					}
				}

				// if it is the ending NAVPack
				if(i == packCount - 1 && pPackBuffer[i]->Packet.streamID == 0xBF)
				{
					overshootNAVPack = 1;
				}
				// else copy packet to target position
				else if(ignore == 0)
				{
					// move packet at the new position
					PESPack *tPack = pPackBuffer[targetPackPos];
					pPackBuffer[targetPackPos] = pPackBuffer[i];
					pPackBuffer[i] = tPack;
					// set next position in write stream
					targetPackPos++;
				}
			}

			// if we have to translate sectors from IFO table
			if(((i < packCount - 1) || overShootSize == 0) && !(overshootNAVPack))
			{
				// do not add -1 here because tsSrcReadSECT and tsTargetWrittenSECT (+VOB) are incremented later (sector count begins from 0)
				// do add -1 to targetPackPos because it is already incremented
				_ASSERT(tsSrcReadSECT + i <= currTransformSECTIFO);
				if(tsSrcReadSECT + i == currTransformSECTIFO)
				{
					pIFO->SetSectorEntry(currTransformSECTIFO, tsTargetWrittenSECT    + targetPackPos - 1, REFERENCE_IFO); // -0 because sector count begins from 0 (incremented later) and
																													 // -1 because targetPackPos begins from 1 at this point
					int x = pIFO->GetNextSectorEntry(REFERENCE_IFO);
					currTransformSECTIFO = x;
				}

				_ASSERT(tsSrcReadSECTVOB + i <= currTransformSECTVOB);
				if(tsSrcReadSECTVOB + i == currTransformSECTVOB)
				{
					pIFO->SetSectorEntry(currTransformSECTVOB, tsTargetWrittenSECTVOB + targetPackPos - 1, VOBnum == 0 ? REFERENCE_MENU_VOB : REFERENCE_TITLE_VOB); // -0 because sector count begins from 0 (incremented later) and
																																								  // -1 because targetPackPos begins from 1 at this point
					int x = pIFO->GetNextSectorEntry(VOBnum == 0 ? REFERENCE_MENU_VOB : REFERENCE_TITLE_VOB);
					currTransformSECTVOB = x;
				}
			}
		}

		// if there is still data to copy
		while(dataSize != dataPos)
		{
			_ASSERT(FALSE);
			// move packet at the new position
			PESPack *tPack = pPackBuffer[targetPackPos];
			pPackBuffer[targetPackPos] = pPackBuffer[i];
			pPackBuffer[i] = tPack;

			// calculate new payload size
			int tSize = pPackBuffer[targetPackPos]->Packet.maxPayLoadSize;
			int bytesToCopy = min(tSize, dataSize - dataPos);
			// set new payload size
			pPackBuffer[targetPackPos]->Packet.SetPayLoadSize(bytesToCopy);
			// copy data into payload
			CopyMemory(pPackBuffer[targetPackPos]->Packet.payLoad, &pData[dataPos], bytesToCopy);
			dataPos += bytesToCopy;

			// set next position in write stream
			targetPackPos++;
		}
//	}
	// if dataSize >= seqSize
	// if there is now more data than it was before then do nothing but copy packets
/*	else
	{
		for(i = 0; i < packCount; i++)
		{
			targetPackPos++;
			// if we have to translate sectors from IFO table
			if(tsSrcReadSECT + i == currTransformSECT)
			{
				pIFO->SetSectorEntry(currTransformSECT, tsTargetWrittenSECT + targetPackPos);
				currTransformSECT = pIFO->GetNextSectorEntry();
			}
		}

		// check for sequence header in the middle of packet
		int posSeq = SearchLastSequenceHeader(pPackBuffer[packCount - 1]);
		// set new charge size
		pPackBuffer[targetPackPos]->Packet.charge_size = posSeq;
	}*/

	wasVideoBefore = 0;
	int packetsFinalized = 0;
	// write stream to disk
	for(i = 0; i < targetPackPos; i++)
	{
		bool cond = pPackBuffer[i]->category == CATEGORY_SEQUENCE_HEADER && wasVideoBefore;
		// if overshoot with sequence (sequence header after video packets detected)
		if(cond)
		{
			// move pack at the beginning
			_ASSERT(i == targetPackPos - 1);
			PESPack *tPack = pPackBuffer[0];
			pPackBuffer[0] = pPackBuffer[i];
			pPackBuffer[i] = tPack;
			wasVideoBefore = 2;	// mark end of sequence
		}

		if(pPackBuffer[i]->Packet.streamID >= 0xE0 && pPackBuffer[i]->Packet.streamID <= 0xEF && wasVideoBefore != 2) wasVideoBefore = 1;

		// queue packet for writing if we have no overshoot NAVPack
		if(!cond)
		{
			PESPack *tPack = pPackBufferFinalized[packetsWriteWait];
			pPackBufferFinalized[packetsWriteWait] = pPackBuffer[i];
			pPackBuffer[i] = tPack;

			packetsFinalized++;
			packetsWriteWait++;
		}
	}

	// move pack at the beginning (if end of sequence not marked by compressed packets)
	if(wasVideoBefore != 2 && wasVideoBefore && (pPackBuffer[packCount - 1]->category == CATEGORY_SEQUENCE_HEADER))
	{
		wasVideoBefore = 2;	// mark end of sequence
		PESPack *tPack = pPackBuffer[0];
		pPackBuffer[0] = pPackBuffer[packCount - 1];
		pPackBuffer[packCount - 1] = tPack;
		pPackBuffer[packCount - 1]->Packet.streamID = 0x00; // destroy streamID in order not to mix up with end NAVPack
	}

	// set bitrate control output statistics
	VideoSeq.BitRateControl.outputBytes += packetsFinalized * pPackBuffer[0]->SECTOR_SIZE;
	VideoSeq.BitRateControl.outputBytesSeq = packetsFinalized * pPackBuffer[0]->SECTOR_SIZE;

	// count read and written sectors for IFO table
	tsSrcReadSECT += packCount - overshootNAVPack - ((wasVideoBefore == 2) ? 1 : 0);
	tsTargetWrittenSECT += packetsFinalized;

	tsSrcReadSECTVOB += packCount - overshootNAVPack - ((wasVideoBefore == 2) ? 1 : 0);
	tsTargetWrittenSECTVOB += packetsFinalized;

	// move NAVPack to the beginning for the next time
	// this condition excludes the first moving to forward condition ealier on
	if(pPackBuffer[packCount - 1]->Packet.streamID == 0xBF)
	{
		PESPack *tPack = pPackBuffer[0];
		pPackBuffer[0] = pPackBuffer[packCount - 1];
		pPackBuffer[packCount - 1] = tPack;		
		packCount = 1;
	}
	else
	{
		packCount = 0;
	}

	return 1;
}

int Multiplexer::ProcessVOBU()
{
	int i;
	int ret; 

	// while there are sequences in the VOBU
	while((ret = ReadSequence()) == 1) WriteSequence();

	WriteSequence();

	// renew indexes for references frames
	int refFrame[3] = { 0, 0, 0 };
	int refFrameIdx = 0;
	int prevType = 0;

	// get the end bytes of the three reference frames
	for(i = 0; i < VideoSeq.stPosLUT.numEntries; i++)
	{
		int type = VideoSeq.stPosLUT.pEntries[i].entryType;
		if((prevType == I_PICTURE || prevType == P_PICTURE) &&
			(type == I_PICTURE || type == P_PICTURE || type == B_PICTURE))
		{
			// do not count the first I_PICTURE
//				if(prevType == 0) prevType = -1;
//				else
//				{
				prevType = type;
				if(refFrameIdx < 3)
				{
					refFrame[refFrameIdx++] = VideoSeq.stPosLUT.pEntries[i].pos;
				}
//				}
			if(refFrameIdx >= 3) break;
		}
		if(type == I_PICTURE || type == P_PICTURE || type == B_PICTURE) prevType = type;
	}

	refFrameIdx = 0;
	int currBytes = 0;

	// get the end blocks of the three reference frames
	pNAV[0].AttachPESPacket(&pPackBufferFinalized[0]->Packet);
	for(i = 0; i < packetsWriteWait; i++)
	{
		if(pPackBufferFinalized[i]->Packet.streamID >= 0xE0 && pPackBufferFinalized[i]->Packet.streamID <= 0xEF)
		{
			currBytes += pPackBufferFinalized[i]->Packet.payLoadSize;
			while(currBytes >= refFrame[refFrameIdx] && refFrameIdx < 3)
			{
				pNAV[0].SetFrameEndSECT(i, refFrameIdx++);
			}
		}
		if(refFrameIdx >= 3) break;
	}

	// write down packets
	for(i = 0; i < packetsWriteWait; i++)
	{
		pPackBufferFinalized[i]->Write(&WriteIO);
	}

	packetsWriteWait = 0;

	// if there is nothing to read or some other errors
	if(ret < 0) return ret;

	return 0;
}

int Multiplexer::SearchEncryption(IOLayer *ReadIO)
{
	while(true)
	{
		int ret = pPackBuffer[0]->Read(ReadIO);

		// if encryption found
		if(ret == -2) return -1;
		// if no encryption or video found
		if(ret < 0) return 0;
		if(pPackBuffer[0]->Packet.streamID >= 0xE0 && pPackBuffer[0]->Packet.streamID <= 0xEF) return 0;
	}
}

void Multiplexer::SetIgnoreTable(STREAM_IGNORETABLE *IgnoreTable)
{
	this->IgnoreTable = IgnoreTable;
}

void Multiplexer::SetQuality(int quality)
{
	this->quality = quality;
	VideoSeq.BitRateControl.SetQuality(quality);
}

// forces the target size to be 99% (force=1), instead of 95% (0) of the target size
void Multiplexer::ForceTargetSize(int force)
{
	VideoSeq.BitRateControl.ForceTargetSize(force);
}										

// sets a pointer to an alert object
// the alert object can be changed at will before and after setting it in transfer
void Multiplexer::SetAlert(TransferAlert *alert)
{
	this->alert = alert;
	VideoSeq.SetAlert(alert);
}