/*****************************************************************************
 * Picture.cpp: Rebuild Picture Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "picture.h"

Picture::Picture(void)
{
	internal_width = 0;
	internal_height = 0;
	internal_dest_width = 0;
	internal_dest_height = 0;

	// maximum sample picture height(*8) and width(*8)
	max_width = 1272;
	max_height = 576;

	sliceSkipProb = 0;
	previewPicture = 0;
}

Picture::~Picture(void)
{
	if(internal_width != 0) 
	{
		delete DCYUVBitmap[0];
		delete DCYUVBitmap[1];
		delete DCYUVBitmap[2];

		delete DCYUVBitmapAR[0];
		delete DCYUVBitmapAR[1];
		delete DCYUVBitmapAR[2];

		delete DCRGBBitmap;
	}
}

// attach new buffer to the object
int Picture::AttachBuffer(unsigned char *buffer, int length, unsigned char *wbuffer, int posLUTindex)
{
	// initialize parameters
	pBuffer = buffer;
	bufferSize = length;

	this->posLUTindex = posLUTindex;

	// attach a bitstream to the buffer
	bstream.AttachBuffer(pBuffer, bufferSize);

	// calc aspect ratio corrected width and height
	dest_width = width; dest_height = height;
	switch(aspect_ratio)
	{
		case 2: if(((float)width / height) < 4.0 / 3.0)  dest_width = width * 4.0 / 3.0  / ((float)width / height); else dest_height = height * 3.0 / 4.0  * ((float)width / height); break;
		case 3: if(((float)width / height) < 16.0 / 9.0) dest_width = width * 16.0 / 9.0 / ((float)width / height); else dest_height = height * 9.0 / 16.0 * ((float)width / height); break;
		case 4: if(((float)width / height) < 2.21)       dest_width = width * 2.21       / ((float)width / height); else dest_height = height / 2.21       * ((float)width / height); break;
	}

	int val;
	// read picture header
	// read picture header start code
	val = bstream.ReadBits(32);
	_ASSERTX(val == 0x00000100);

	// *** read Parameters

	// not needed now
	bstream.ReadBits(10);

	pictureType = bstream.ReadBits(3);

	// not needed now
	bstream.ReadBits(16);
	if(pictureType == P_PICTURE || pictureType == B_PICTURE) bstream.ReadBits(4);
	if(pictureType == B_PICTURE) bstream.ReadBits(4);
	if(bstream.ReadBit() == 1)
	{
		bstream.ReadBits(8);
	}

	// read picture coding extension
	// read extension start code
	bstream.SeekNextByte();
	val = bstream.MPEG2_GetNextStartCode();
	_ASSERTX(val == 0x000001B5);

	// read picture coding extension identifier
	val = bstream.ReadBits(4);
	_ASSERTX(val == 8);

	f_code[0][0] = bstream.ReadBits(4);
	f_code[0][1] = bstream.ReadBits(4);
	f_code[1][0] = bstream.ReadBits(4);
	f_code[1][1] = bstream.ReadBits(4);

	intraDCPrecision = bstream.ReadBits(2);
	
	picture_structure = bstream.ReadBits(2);

	// not needed now
	bstream.ReadBits(1);

	frame_pred_frame_dct = bstream.ReadBits(1);

	concealmentMotionVectors = bstream.ReadBit();
	qScaleType = bstream.ReadBit();
	intraVLCFormat = bstream.ReadBit();
	alternateScan =  bstream.ReadBit();

	// not needed now
	bstream.ReadBits(3);
	if(bstream.ReadBit())
	{
		bstream.ReadBits(20);
	}

	absNumMacroBlock = 0;

	relPos = stPosLUT->pEntries[posLUTindex].pos;

	// ****** BEGIN write
	int windex = posLUTindex;

	// move header
	int wlength = FindNextSliceStartCode(&windex) - relPos;
	MoveMemory(wbuffer, pBuffer, wlength);
	// ****** END write

	// ****** BEGIN preview
	unsigned char *Y, *U, *V;
	int DCCoef[12];
	int XY[4];
	signed char shift;
	int Upos[4], Vpos[4];
	if(pictureType == I_PICTURE && previewPicture)
	{
		ReallocPreviewBitmaps();
	
		if(chroma_format == CHROMA_420) { Upos[0] = 4; Upos[1] = 4; Upos[2] = 4; Upos[3] = 4; Vpos[0] = 5; Vpos[1] = 5; Vpos[2] = 5;  Vpos[3] = 5;  }
		if(chroma_format == CHROMA_422) { Upos[0] = 4; Upos[1] = 4; Upos[2] = 5; Upos[3] = 5; Vpos[0] = 6; Vpos[1] = 6; Vpos[2] = 7;  Vpos[3] = 7;  }
		if(chroma_format == CHROMA_444) { Upos[0] = 4; Upos[1] = 5; Upos[2] = 6; Upos[3] = 7; Vpos[0] = 8; Vpos[1] = 9; Vpos[2] = 10; Vpos[3] = 11; }

		// shift to fit unsigned char
		shift = 3 - intraDCshift[intraDCPrecision];

		Y = DCYUVBitmap[0];
		U = DCYUVBitmap[1];
		V = DCYUVBitmap[2];
	}
	// ****** END preview

	// control the bitrate (adjust compression parameters) of the sequence
	BitRateControl->ProcessPicture(this);
	Slices.BitRateControl = BitRateControl;
	
	int i;
	int index = posLUTindex;
	numSlices = CountSlices();
	// if there are slices
	for(i = 0; i < numSlices/* - 1*/; i++)
	{
		// find picture start code
		int slicePos = FindNextSliceStartCode(&index) - relPos;

		// find next picture start code
		int savIndex = index;
		int savSlicePos = slicePos;
		slicePos = FindNextSliceStartCode(&index) - relPos;

		// calc offset
		int offset;
		if(slicePos + relPos != -1) offset = slicePos - savSlicePos;
		else						offset = bufferSize - savSlicePos;

		if(i == numSlices - 1)
		{
			offset = offset;
		}

		index = savIndex;
		slicePos = savSlicePos;
		int skippedSlice = 0;

//		_ASSERT(pictureType != I_PICTURE || (stPosLUT->pEntries[savIndex].skip == 0 && pictureType == I_PICTURE));
		if(stPosLUT->pEntries[savIndex].skip == 0/* rand() % MAX_SLICE_SKIP_PROB > sliceSkipProb/* || pictureType == I_PICTURE*/) // do some improvements here, there must be something else than random
		{
			SetSliceData(&Slices);
			wlength += Slices.AttachBuffer(&pBuffer[slicePos], offset, wbuffer + wlength);
			absNumMacroBlock = Slices.absNumMacroBlock;
		}
		else
		{
			CopyMemory(wbuffer + wlength, &pBuffer[slicePos], offset);
			wlength += offset;
			// estimate number of macroblocks to add to absNumMacroBlock
			absNumMacroBlock += width / 16;
			skippedSlice = 1;
		}

		if(pictureType == I_PICTURE && previewPicture)
		{
			// if slice was skipped we do have to recover it
			if(skippedSlice)
			{
				SetSliceData(&Slices);
				Slices.AttachBuffer(&pBuffer[slicePos], offset, wbuffer + wlength);
			}
			// ****** BEGIN preview
			if(pictureType == I_PICTURE && previewPicture) 
			{
				XY[0] = (i * 2)     * width / 8; XY[1] = XY[0] + 1;
				XY[2] = (i * 2 + 1) * width / 8; XY[3] = XY[2] + 1;

				for(int j = 0; j < Slices.currMB; j++)
				{
					Slices.MB[j]->GetDCCoef(DCCoef);
					Y[XY[0]] = DCCoef[0] >> shift;
					Y[XY[1]] = DCCoef[1] >> shift;
					Y[XY[2]] = DCCoef[2] >> shift;
					Y[XY[3]] = DCCoef[3] >> shift;

					U[XY[0]] = DCCoef[Upos[0]] >> shift;
					U[XY[1]] = DCCoef[Upos[1]] >> shift;
					U[XY[2]] = DCCoef[Upos[2]] >> shift;
					U[XY[3]] = DCCoef[Upos[3]] >> shift;

					V[XY[0]] = DCCoef[Vpos[0]] >> shift;
					V[XY[1]] = DCCoef[Vpos[1]] >> shift;
					V[XY[2]] = DCCoef[Vpos[2]] >> shift;
					V[XY[3]] = DCCoef[Vpos[3]] >> shift;

					XY[0] += 2; XY[1] += 2; XY[2] += 2; XY[3] += 2;
				}
			}
			// ****** END preview
		}
	}
	/*int slicePos = FindNextSliceStartCode(&index) - relPos;
	SetSliceData(&Slices);
	wlength += Slices.AttachBuffer(&pBuffer[slicePos], bufferSize - slicePos, wbuffer + wlength);*/

	// ****** BEGIN preview
	if(pictureType == I_PICTURE && previewPicture) GenerateDCPicture();
	// ****** END preview

	return wlength;
}

int Picture::intraDCshift[4] = { 3, 2, 1, 0 };

void Picture::ReallocPreviewBitmaps()
{
	int padding_32b = (4 - ((max_width / 8) * 3) % 4) % 4;

	if(internal_width != width || internal_height != height || 
	   internal_dest_width != dest_width || internal_dest_height != dest_height) 
	{
		if(internal_width != 0)
		{
			delete DCYUVBitmap[0];
			delete DCYUVBitmap[1];
			delete DCYUVBitmap[2];

			delete DCYUVBitmapAR[0];
			delete DCYUVBitmapAR[1];
			delete DCYUVBitmapAR[2];

			delete DCRGBBitmap;
		}

		internal_width = width;
		internal_height = height;
		internal_dest_width = dest_width;
		internal_dest_height = dest_height;

		DCYUVBitmap[0] = new unsigned char[height * width / 64];
		DCYUVBitmap[1] = new unsigned char[height * width / 64];
		DCYUVBitmap[2] = new unsigned char[height * width / 64];

		DCYUVBitmapAR[0] = new unsigned char[dest_height * dest_width / 64];
		DCYUVBitmapAR[1] = new unsigned char[dest_height * dest_width / 64];
		DCYUVBitmapAR[2] = new unsigned char[dest_height * dest_width / 64];
		
		DCRGBBitmap    = new unsigned char[(max_height / 8) * (max_width / 8) * 3 + (max_height / 8) * padding_32b];
	}
}

void Picture::GenerateDCPicture()
{
	int padding_32b = (4 - ((max_width / 8) * 3) % 4) % 4;

	unsigned char *Y = DCYUVBitmap[0];
	unsigned char *U = DCYUVBitmap[1];
	unsigned char *V = DCYUVBitmap[2];

	unsigned char *Yar = DCYUVBitmapAR[0];
	unsigned char *Uar = DCYUVBitmapAR[1];
	unsigned char *Var = DCYUVBitmapAR[2];

	if(width != dest_width || height != dest_height)
	{
		if(width < dest_width) 
		{
			float width_corr_step = (float)width / (dest_width - width);
			for(int i = 0; i < height / 8; i++)
			{
				int corr = 0;
				float fcorr = 0;
				unsigned char *Yarline = &Yar[i * dest_width / 8];
				unsigned char *Uarline = &Uar[i * dest_width / 8];
				unsigned char *Varline = &Var[i * dest_width / 8];
				unsigned char *Yline = &Y[i * width / 8];
				unsigned char *Uline = &U[i * width / 8];
				unsigned char *Vline = &V[i * width / 8];
				for(int j = 0; j < width / 8; j++)
				{
					// if a gap is to be filled, interpolate
					if(j >= fcorr + width_corr_step) 
					{ 
						Yarline[j + corr] = (Yline[j - 1] + Yline[j]) / 2;
						Uarline[j + corr] = (Uline[j - 1] + Uline[j]) / 2;
						Varline[j + corr] = (Vline[j - 1] + Vline[j]) / 2;
						corr++; 
						fcorr += width_corr_step;
					}
					Yarline[j + corr] = Yline[j];
					Uarline[j + corr] = Uline[j];
					Varline[j + corr] = Vline[j];
				}
				// if last sample also has to be interpolated
				if(width / 8 + corr < dest_width / 8)
				{
					// move last stored sample
					Yarline[dest_width / 8 - 1] = Yline[width / 8 - 1];
					Uarline[dest_width / 8 - 1] = Uline[width / 8 - 1];
					Varline[dest_width / 8 - 1] = Vline[width / 8 - 1];

					// interpolate new sample
					Yarline[dest_width / 8 - 2] = (Yline[width / 8 - 2] + Yline[width / 8 - 1]) / 2;
					Uarline[dest_width / 8 - 2] = (Uline[width / 8 - 2] + Uline[width / 8 - 1]) / 2;
					Varline[dest_width / 8 - 2] = (Vline[width / 8 - 2] + Vline[width / 8 - 1]) / 2;
				}
			}
		}
		if(height < dest_height) 
		{
			float height_corr_step = (float)height / (dest_height - height);
			int corr = 0;
			float fcorr = 0;
			for(int i = 0; i < height / 8; i++)
			{
				if(i >= fcorr + height_corr_step) 
				{ 
					unsigned char *Yarline = &Yar[(i + corr) * dest_width / 8];
					unsigned char *Uarline = &Uar[(i + corr) * dest_width / 8];
					unsigned char *Varline = &Var[(i + corr) * dest_width / 8];
					unsigned char *Yline = &Y[i * width / 8];
					unsigned char *Uline = &U[i * width / 8];
					unsigned char *Vline = &V[i * width / 8];
					for(int j = 0; j < width / 8; j++)
					{
						Yarline[j] = (Yline[j - width / 8] + Yline[j]) / 2;
						Uarline[j] = (Uline[j - width / 8] + Uline[j]) / 2;
						Varline[j] = (Vline[j - width / 8] + Vline[j]) / 2;
					}
					corr++; 
					fcorr += height_corr_step;
				}
				unsigned char *Yarline = &Yar[(i + corr) * dest_width / 8];
				unsigned char *Uarline = &Uar[(i + corr) * dest_width / 8];
				unsigned char *Varline = &Var[(i + corr) * dest_width / 8];
				unsigned char *Yline = &Y[i * width / 8];
				unsigned char *Uline = &U[i * width / 8];
				unsigned char *Vline = &V[i * width / 8];
				for(int j = 0; j < width / 8; j++)
				{
					Yarline[j] = Yline[j];
					Uarline[j] = Uline[j];
					Varline[j] = Vline[j];
				}
			}
		}
	}
	else
	{
		CopyMemory(Yar, Y, width * height / 64);
		CopyMemory(Uar, U, width * height / 64);
		CopyMemory(Var, V, width * height / 64);
	}

	// clear bitmap
	ZeroMemory(DCRGBBitmap, (max_height / 8) * (max_width / 8) * 3 + (max_height / 8) * padding_32b);

	// copy bitmap into right place
	int rowLength = (max_width / 8) * 3 + padding_32b;
	int xoff = ((max_width / 8) - (dest_width / 8)) / 2;
	int yoff = ((max_height / 8) - (dest_height / 8)) / 2;
	for(int i = 0; i < dest_height / 8; i++)
	{
		unsigned char *Yarline = &Yar[i * dest_width / 8];
		unsigned char *Uarline = &Uar[i * dest_width / 8];
		unsigned char *Varline = &Var[i * dest_width / 8];
		unsigned char *RGBline = &DCRGBBitmap[(i + yoff) * rowLength + xoff * 3];
		for(int j = 0; j < dest_width / 8; j++)
		{
			int Y = Yarline[j];
			int U = Uarline[j] - 128;
			int V = Varline[j] - 128;
			int R = Y + ((1167 * V) >> 10);
			int G = Y - ((403 * U + 595 * V) >> 10);
			int B = Y + ((2081 * U) >> 10);
			RGBline[j * 3    ] = B < 0 ? 0 : (B > 255 ? 255 : B);
			RGBline[j * 3 + 1] = G < 0 ? 0 : (G > 255 ? 255 : G);
			RGBline[j * 3 + 2] = R < 0 ? 0 : (R > 255 ? 255 : R);
		}
	}

}

// returns the the dc bitmap
unsigned char *Picture::GetDCPicture()
{
	return DCRGBBitmap;
}

// find the next picture start code in the buffer
int Picture::FindNextSliceStartCode(int *index)
{
	// get NEXT picture
	(*index)++;

	// while we are not at the end of the table
	while(*index < stPosLUT->numEntries)
	{
		int type = stPosLUT->pEntries[*index].entryType;
		if(type == SLICE)
		{
			return stPosLUT->pEntries[*index].pos;
		}
		// if we have arrived at the next picture return nothing found
		if(type == I_PICTURE || type == P_PICTURE || type == B_PICTURE)
		{
			return -1;
		}
		// next entry
		(*index)++;
	}

	// return nothing found
	return -1;
}

int Picture::CountSlices()
{
	// init index
	int index = posLUTindex + 1;
	int numSlices = 0;

	// while we are not at the end of the table
	while(index < stPosLUT->numEntries)
	{
		int type = stPosLUT->pEntries[index].entryType;
		if(type == SLICE)
		{
			numSlices++;
		}
		// if we have arrived at the next picture return
		if(type == I_PICTURE || type == P_PICTURE || type == B_PICTURE)
		{
			break;
		}
		// next entry
		index++;
	}

	// return number of slices found
	return numSlices;
}

// *** last version
// find the next picture start code in the buffer
/*int Picture::FindNextSliceStartCode(int pos)
{
	int currPos = pos + 1;

	// while we are not at the end of the buffer
	while(currPos + 4 < bufferSize)
	{
		if(pBuffer[currPos + 2] > 0x01)
		{
			currPos += 3;
		}
		else if(pBuffer[currPos + 1] > 0x01)
		{
			currPos += 2;
		}
		else if(pBuffer[currPos + 3] > 0xAF)
		{
			currPos += 4;
		}
		else if(pBuffer[currPos] == 0x00 && pBuffer[currPos + 1] == 0x00 && pBuffer[currPos + 2] == 0x01 && pBuffer[currPos + 3] >= 0x01 && pBuffer[currPos + 3] <= 0xAF)
		{
			return currPos - pos;
		}
		else 
		{
			currPos++;
		}
	}

	// return nothing found
	return -1;
}*/

// last thouroughly tested version
// find the next picture start code in the buffer
/*int Picture::FindNextSliceStartCode(int pos)
{
	int currPos = pos + 1;

	// while we are not at the end of the buffer
	while(currPos + 4 < bufferSize)
	{
		int currVal = (pBuffer[currPos]     << 24) + 
			          (pBuffer[currPos + 1] << 16) + 
					  (pBuffer[currPos + 2] <<  8) + 
					  (pBuffer[currPos + 3]      );
		// if start code found return
		if(currVal >= 0x00000101 && currVal <= 0x000001AF)
		{
			return currPos - pos;
		}
		// next byte	
		currPos++;
	}

	// return nothing found
	return -1;
}*/

int Picture::SetSliceData(Slice *pSlice)
{

	pSlice->f_code[0][0] = f_code[0][0];
	pSlice->f_code[0][1] = f_code[0][1];
	pSlice->f_code[1][0] = f_code[1][0];
	pSlice->f_code[1][1] = f_code[1][1];
	pSlice->pictureType = pictureType;
	pSlice->intraDCPrecision = intraDCPrecision;
	pSlice->picture_structure = picture_structure;
	pSlice->frame_pred_frame_dct = frame_pred_frame_dct;
	pSlice->concealmentMotionVectors = concealmentMotionVectors;
	pSlice->qScaleType = qScaleType;
	pSlice->intraVLCFormat = intraVLCFormat;
	pSlice->alternateScan = alternateScan;
	pSlice->chroma_format = chroma_format;
	pSlice->absNumPicture = absNumPicture;
	pSlice->absNumMacroBlock = absNumMacroBlock;
	pSlice->demoMode = demoMode;

	return TRUE;
}