/*****************************************************************************
 * VideoSequence.cpp: Video Buffer Handling Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "Transfer.h"
#include "videosequence.h"
#include "resource.h"

VideoSequence::VideoSequence(void)
{
	numPictures = 12;
	skipped = 0;
	absNumIPicture = 0;
	previewPicture = 1;

	stPosLUT.maxNumEntries = 2 * 12 * 72;
	stPosLUT.pEntries = new STARTCODE_POS_ENTRY[stPosLUT.maxNumEntries];
}

VideoSequence::~VideoSequence(void)
{
	delete stPosLUT.pEntries;
}

// attach a buffer to the object
int VideoSequence::AttachBuffer(unsigned char *buffer, int length, unsigned char *wbuffer)
{
	pBuffer = buffer;
	bufferSize = length;

	numPictures = 0;
	skipped = 0;

	// fill start code LUT and if sequence is not transferable by this program then return
	if(!FillStartCodePosLUT()) 
	{
		skipped = 1;
		return -1;
	}

	// attach bitstream to new buffer
	bstream.AttachBuffer(pBuffer, bufferSize);

	int val, i;

	// init parameters
/*	SetDefaultIntraMatrix();
	SetDefaultInterMatrix();*/

	// read sequence header
	val = bstream.ReadBits(32);
	_ASSERTX(val == 0x000001B3 || val == 0x000000B3);	// mysterious 0x000000B3 seems to be valid although not in specification

	// not needed now
//	bstream.Seek(62);
	width  = bstream.ReadBits(12);
	height = bstream.ReadBits(12);
	aspect_ratio = bstream.ReadBits(4);
	bstream.Seek(34);

	// read intra quantizer matrix flag and matrix
	val = bstream.ReadBit();
//	if(val) LoadIntraMatrix(); 
	if(val) bstream.Seek(512);

	// read inter quantizer matrix flag and matrix
	val = bstream.ReadBit();
//	if(val) LoadInterMatrix();
	if(val) bstream.Seek(512);

	// next must be a sequence extension header
	// read extension start code
	val = bstream.ReadBits(32);
	_ASSERTX(val == 0x000001B5);
	// read sequence extension identifier
	val = bstream.ReadBits(4);
	_ASSERTX(val == 1);

	// not needed now
	bstream.Seek(9);

	chroma_format = bstream.ReadBits(2);

	// not needed now
	bstream.Seek(33);

	// search for group start code
	do
	{
		val = bstream.MPEG2_GetNextStartCode();
		
	} while(val != 0x000001B8 && val != -1);
	_ASSERTX(val == 0x000001B8);

	// not needed now
	bstream.Seek(32);

	// ****** BEGIN write
	int windex = 0;
	// move header
	int wlength = FindNextPictureStartCode(&windex);
	MoveMemory(wbuffer, pBuffer, wlength);
	// ****** END write

	int index = 0;
	numPictures = numIPictures + numPPictures + numBPictures;

	// process the pictures
	Frame.chroma_format = chroma_format;
	Frame.width = width;
	Frame.height = height;
	Frame.aspect_ratio = aspect_ratio;
	Frame.demoMode = demoMode;
	Frame.stPosLUT = &stPosLUT;
	Frame.BitRateControl = &BitRateControl;

	BitRateControl.UpdateStatistics();
	Strategy.CreateStrategy(&stPosLUT, length, numIPictures, numPPictures, numBPictures, min(BitRateControl.GetSliceSkipProb() * length, length));

	// if there are pictures
	for(i = 0; i < numPictures; i++)
	{
		// find picture start code
		int picturePos = FindNextPictureStartCode(&index);

		// find next picture start code
		int savIndex = index;
		int savPicturePos = picturePos;
		picturePos = FindNextPictureStartCode(&index);

		// calc offset
		int offset;
		if(picturePos != -1) offset = picturePos - savPicturePos;
		else				 offset = bufferSize - savPicturePos;

		index = savIndex;
		picturePos = savPicturePos;

		Frame.absNumPicture = absNumPicture;
		Frame.previewPicture = previewPicture;

		wlength += Frame.AttachBuffer(&pBuffer[picturePos], offset, wbuffer + wlength, savIndex);
		stPosLUT.pEntries[savIndex].pos = wlength;

		if(Frame.pictureType == I_PICTURE)
		{
			if(alert != NULL && previewPicture)
			{
				if(alert->GetAlert(ALERT_PREVIEW_AVAIL)) 
				{
					HBITMAP hBmp = GenerateDCPicture(alert->GetPreviewDC());
					if(alert->SendAlert(ALERT_PREVIEW_AVAIL, (int)hBmp) <= 0) DeleteObject(hBmp);
				}
			}

			// see whether the next I picture should be a preview picture
			absNumIPicture++;
			if(absNumIPicture % 4 == 0) previewPicture = 1;
			else					    previewPicture = 0;
		}
//			_RPT1(_CRT_WARN, "Sequence %d\n", numPictures);
		picturePos += offset;
		absNumPicture++;
	}

	// control the bitrate (adjust compression parameters) of the sequence
//	BitRateControl.ProcessFrame(this);

/*	// write sequence end code
	buffer[length    ] = 0x00;
	buffer[length + 1] = 0x00;
	buffer[length + 2] = 0x01;
	buffer[length + 3] = 0xB7;
	length += 4;*/

	return wlength;
}

/*// default intra quantizer matrix
const unsigned char VideoSequence::default_intra_matrix[64] =
{ 8, 16, 19, 22, 26, 27, 29, 34,
 16, 16, 22, 24, 27, 29, 34, 37,
 19, 22, 26, 27, 29, 34, 34, 38,
 22, 22, 26, 27, 29, 34, 37, 40,
 22, 26, 27, 29, 32, 35, 40, 48,
 26, 27, 29, 32, 35, 40, 48, 58,
 26, 27, 29, 34, 38, 46, 56, 69,
 27, 29, 35, 38, 46, 56, 69, 83 };

// zig-zag and alternate scan patterns 
const unsigned char VideoSequence::scan[2][64] = 
{
	{ // Zig-Zag scan pattern
		0,  1,  8, 16,  9,  2,  3, 10,
	   17, 24, 32, 25, 18, 11,  4,  5,
	   12, 19, 26, 33, 40, 48, 41, 34,
	   27, 20, 13,  6,  7, 14, 21, 28,
	   35, 42, 49, 56, 57, 50, 43, 36,
	   29, 22, 15, 23, 30, 37, 44, 51,
	   58, 59, 52, 45, 38, 31, 39, 46,
	   53, 60, 61, 54, 47, 55, 62, 63
	}
	,
	{ // Alternate scan pattern
		0,  8, 16, 24,  1,  9,  2, 10,
	   17, 25, 32, 40, 48, 56, 57, 49,
	   41, 33, 26, 18,  3, 11, 4,  12,
	   19, 27, 34, 42, 50, 58, 35, 43,
	   51, 59, 20, 28,  5, 13,  6, 14,
	   21, 29, 36, 44, 52, 60, 37, 45,
	   53, 61, 22, 30,  7, 15, 23, 31,
	   38, 46, 54, 62, 39, 47, 55, 63
	}
};

// loads the default intra quantizer matrix
void VideoSequence::SetDefaultIntraMatrix(void)				 
{ 
	for (int i = 0; i < 64; i++) 
	{
		intra_qmatrix[i] = default_intra_matrix[i];
		chroma_intra_qmatrix[i] = default_intra_matrix[i];
	}
}

// loads the default inter quantizer matrix
void VideoSequence::SetDefaultInterMatrix(void)
{
    for (int i = 0; i < 64; i++) 
	{
		non_intra_qmatrix[i] = 16;
		chroma_non_intra_qmatrix[i] = 16;
	}
}

// loads an intra quantizer matrix from the stream
void VideoSequence::LoadIntraMatrix(void)				 
{ 
	for (int i = 0; i < 64; i++) 
	{
		int pos = scan[0][i];
		int val = bstream.ReadBits(8);
		intra_qmatrix[pos] = val;
		chroma_intra_qmatrix[pos] = val;
	}
}

// loads an inter quantizer matrix from the stream
void VideoSequence::LoadInterMatrix(void)
{
    for (int i = 0; i < 64; i++) 
	{
		int pos = scan[0][i];
		int val = bstream.ReadBits(8);
		non_intra_qmatrix[pos] = val;
		chroma_non_intra_qmatrix[pos] = val;
	}
}
*/

// find the next picture start code in the buffer
int VideoSequence::FindNextPictureStartCode(int *index)
{
	// get NEXT picture
	(*index)++;

	// while we are not at the end of the table
	while(*index < stPosLUT.numEntries)
	{
		int type = stPosLUT.pEntries[*index].entryType;
		if(type == I_PICTURE || type == P_PICTURE || type == B_PICTURE)
		{
			return stPosLUT.pEntries[*index].pos;
		}
		// next entry
		(*index)++;
	}

	// return nothing found
	return -1;
}

// *** last known good woking version
// find the next picture start code in the buffer
/*int VideoSequence::FindNextPictureStartCode(int *pos)
{
	int currPos = pos + 1;

	// while we are not at the end of the buffer
	while(currPos + 4 < bufferSize)
	{
		// if start code found return
		if(pBuffer[currPos    ] == 0x00 && 
		   pBuffer[currPos + 1] == 0x00 && 
		   pBuffer[currPos + 2] == 0x01 && 
		   pBuffer[currPos + 3] == 0x00)
		{
			return currPos - pos;
		}
		// next byte	
		currPos++;
	}

	// return nothing found
	return -1;
}*/

/*int VideoSequence::CheckTransferability()
{
	int numGOPs = 0;
	int numIPictures = 0;
	int numPBPictures = 0;

	// search for GOPs
	// there must be at least one P or B picture
	// or there must be at least 3 I pictures
	int currPos = 0;
	while(currPos + 5 < bufferSize)
	{
		// if start code found
		if(pBuffer[currPos + 3] == 0x00 && 
		   pBuffer[currPos + 2] == 0x01 &&
		   pBuffer[currPos + 1] == 0x00 && 
		   pBuffer[currPos    ] == 0x00)
		{
			int val = (pBuffer[currPos + 5] >> 3) & 3;
			switch(val)
			{
				case 1: numIPictures++;  break;
				case 2:
				case 3: numPBPictures++; break;
			}
			if(numPBPictures > 1) return TRUE;
			if(numIPictures > 3) return TRUE;
		}
		// next byte	
		currPos++;
	}

	return FALSE;
}*/

// fills all the start code pos in the LUT
// *** returns -1 if sequence is not transferable
int VideoSequence::FillStartCodePosLUT()
{
	int currStPos = 0;
	int currPos = 0;
	// some checking for transferability initialisations
	int retval = 0;
	numIPictures = 0;
	numPPictures = 0;
	numBPictures = 0;

	stPosLUT.numEntries = 0;

	// while we are not at the end of the buffer
	while(currPos + 4 < bufferSize)
	{
		if(pBuffer[currPos + 2] > 0x01)
		{
			currPos += 3;
		}
		else if(pBuffer[currPos + 1] > 0x01)
		{
			currPos += 2;
		}
		else if(pBuffer[currPos] == 0x00 && pBuffer[currPos + 1] == 0x00 && pBuffer[currPos + 2] == 0x01)
		{
			// if there is not enough place to put our entries into
			if(currStPos == stPosLUT.maxNumEntries)
			{
				STARTCODE_POS_ENTRY *tpEntries = new STARTCODE_POS_ENTRY[stPosLUT.maxNumEntries * 2];
				CopyMemory(tpEntries, stPosLUT.pEntries, stPosLUT.numEntries * sizeof(STARTCODE_POS_ENTRY));
				delete stPosLUT.pEntries;
				stPosLUT.pEntries = tpEntries;
				stPosLUT.maxNumEntries * 2;
			}

			// check type of start code
			int type = -1;
			
			switch(pBuffer[currPos + 3])
			{
				// first cast is a picture
				case 0x00:
					{
						_ASSERT(currPos + 5 < bufferSize);
						int val = (pBuffer[currPos + 5] >> 3) & 3;
						switch(val)
						{
							case 1: numIPictures++; type = I_PICTURE; break;
							case 2: numPPictures++; type = P_PICTURE; break;
							case 3: numBPictures++; type = B_PICTURE; break;
						}
						if((numPPictures + numBPictures) > 1) retval = 1;
						if(numIPictures > 3) retval = 1;
					}
					break;

				case 0xB3:
					type = SEQUENCE;
					break;

				case 0xB5:
					type = EXTENSION;
					break;

				case 0xB8:
					type = GOP;
					break;

				default:
					if(pBuffer[currPos + 3] >= 0x01 && pBuffer[currPos + 3] <= 0xAF)
					{
						type = SLICE;
					}
					else 
					{
						_ASSERT(FALSE);
					}
					break;
			}

			// put entry into table
			stPosLUT.pEntries[stPosLUT.numEntries].entryType = type;
			stPosLUT.pEntries[stPosLUT.numEntries].pos = currPos;
			stPosLUT.pEntries[stPosLUT.numEntries].skip = 0;
			
			stPosLUT.numEntries++;

			currPos += 4;
		}
		else 
		{
			currPos++;
		}
	}

	return retval;
}

// sets a pointer to an alert object
// the alert object can be changed at will before and after setting it in transfer
void VideoSequence::SetAlert(TransferAlert *alert)
{
	this->alert = alert;
}

HBITMAP VideoSequence::GenerateDCPicture(HDC hDC)
{
	HBITMAP hBMP = 0;

	// if not skipped create bitmap from I_FRAME
	if(!skipped)
	{
		int max_width = Frame.max_width;
		int max_height = Frame.max_height;
		int padding_32b = (4 - ((max_width / 8) * 3) % 4) % 4;

		BITMAPINFO targetBmi;
		BITMAPINFO sourceBmi;

		ZeroMemory(&targetBmi.bmiHeader, sizeof(BITMAPINFOHEADER));
		targetBmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		targetBmi.bmiHeader.biWidth = max_width / 8;
		targetBmi.bmiHeader.biHeight = -(max_height / 8);
		targetBmi.bmiHeader.biPlanes = 1;
		targetBmi.bmiHeader.biBitCount = 24;
		targetBmi.bmiHeader.biCompression = BI_RGB;
		targetBmi.bmiHeader.biSizeImage = (max_width / 8) * (max_height / 8) * 3 + (max_height / 8) * padding_32b;
		targetBmi.bmiHeader.biXPelsPerMeter = 72;
		targetBmi.bmiHeader.biYPelsPerMeter = 72;

		ZeroMemory(&sourceBmi.bmiHeader, sizeof(BITMAPINFOHEADER));
		sourceBmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		sourceBmi.bmiHeader.biWidth = max_width / 8;
		sourceBmi.bmiHeader.biHeight = -(max_height / 8);
		sourceBmi.bmiHeader.biPlanes = 1;
		sourceBmi.bmiHeader.biBitCount = 24;
		sourceBmi.bmiHeader.biCompression = BI_RGB;
		sourceBmi.bmiHeader.biSizeImage = (max_width / 8) * (max_height / 8) * 3 + (max_height / 8) * padding_32b;
		sourceBmi.bmiHeader.biXPelsPerMeter = 72;
		sourceBmi.bmiHeader.biYPelsPerMeter = 72;

		unsigned char *buf = Frame.GetDCPicture();
		hBMP = CreateDIBitmap(hDC, (BITMAPINFOHEADER *)&targetBmi.bmiHeader, CBM_INIT, buf, &sourceBmi, DIB_RGB_COLORS);
		if(hBMP == 0)
		{
			LPVOID lpMsgBuf;
			FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL 
			);
			// Process any inserts in lpMsgBuf.
			// ...
			// Display the string.
			MessageBox( NULL, (LPCTSTR)lpMsgBuf, "Error", MB_OK | MB_ICONINFORMATION );
			// Free the buffer.
			LocalFree( lpMsgBuf );
		}
	}
	else
	{
		hBMP = (HBITMAP)CopyImage(hHyperSpeedBMP, IMAGE_BITMAP, 0, 0, 0);
	}

	return hBMP;
}

// mark the sequence as being skipped
void VideoSequence::SetHyperSpeed()
{
	// send hyperspeed marking if sequence is the first one skipped
	if(skipped == 0 && alert != NULL)
	{
		if(alert->GetAlert(ALERT_PREVIEW_AVAIL))
		{
			skipped = 1;

			HBITMAP hBmp = GenerateDCPicture(alert->GetPreviewDC());
			if(alert->SendAlert(ALERT_PREVIEW_AVAIL, (int)hBmp) <= 0) DeleteObject(hBmp);
		}
	}
}
