/*****************************************************************************
 * MltiTree.cpp: ShrinkTo5 Track Selection TreeView class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "stdafx.h"
#include <windowsx.h>
#include "MltiTree.h"
#include "..\resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////

#ifndef MST_TIMER_PERIOD
#define MST_TIMER_PERIOD	75		//ms
#endif

/////////////////////////////////////////////////////////////////////////////
// CMultiTree

IMPLEMENT_DYNAMIC(CMultiTree, CMultiTree_BASE)

CMultiTree::CMultiTree()
{
	m_bMulti = FALSE;
	m_hSelect = NULL;
	m_bBandLabel = FALSE;
	m_bEmulated = FALSE;
	m_bMultiCheck = true;
	for(int x=0;x<4;x++)
		m_TreeSave[x].ID = 0;
	m_audID = GetText(TREE_1);
	m_subID = GetText(TREE_2);
}

CMultiTree::~CMultiTree()
{
}

#define CTreeCtrl	CMultiTree_BASE
BEGIN_MESSAGE_MAP(CMultiTree, CTreeCtrl)
#undef CTreeCtrl
	//{{AFX_MSG_MAP(CMultiTree)
	ON_WM_LBUTTONDOWN()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_RBUTTONDOWN()
	ON_NOTIFY_REFLECT_EX(TVN_ITEMEXPANDING, OnItemExpanding)
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
	ON_WM_LBUTTONDBLCLK()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMultiTree message handlers
// basteln der Flags-Checkbox ImageList
void CMultiTree::InitBitmaps()
{
	m_imageState.Create(32, 16, TRUE|ILC_COLOR32, 450, 0);

	CBitmap dvd;
	dvd.LoadBitmap(IDB_DVD);
	m_imageState.Add(&dvd,RGB(255,255,255));
	dvd.DeleteObject();

	CBitmap audio;
	audio.LoadBitmap(IDB_AUDIO);
	m_imageState.Add(&audio,RGB(255,255,255));
	audio.DeleteObject();

	CBitmap subtile;
	subtile.LoadBitmap(IDB_SUBTITLE);
	m_imageState.Add(&subtile,RGB(255,255,255));
	subtile.DeleteObject();

	CBitmap menInfo;
	menInfo.LoadBitmap(IDB_MEN);
	m_imageState.Add(&menInfo,RGB(255,255,255));
	menInfo.DeleteObject();

	CBitmap mensub;
	mensub.LoadBitmap(IDB_MENSUB);
	m_imageState.Add(&mensub,RGB(255,255,255));
	mensub.DeleteObject();
	
	CBitmap menaud;
	menaud.LoadBitmap(IDB_MENAUD);
	m_imageState.Add(&menaud,RGB(255,255,255));
	menaud.DeleteObject();

	CBitmap subaus;
	subaus.LoadBitmap(IDB_SUBAUS);
	m_imageState.Add(&subaus,RGB(255,255,255));
	subaus.DeleteObject();

	CBitmap suban;
	suban.LoadBitmap(IDB_SUBAN);
	m_imageState.Add(&suban,RGB(255,255,255));
	suban.DeleteObject();

	CBitmap audaus;
	audaus.LoadBitmap(IDB_AUDAUS);
	m_imageState.Add(&audaus,RGB(255,255,255));
	audaus.DeleteObject();

	CBitmap audan;
	audan.LoadBitmap(IDB_AUDAN);
	m_imageState.Add(&audan,RGB(255,255,255));
	audan.DeleteObject();

	CBitmap rootitem;
	rootitem.LoadBitmap(IDB_ROOT);
	m_imageState.Add(&rootitem,RGB(255,255,255));
	rootitem.DeleteObject();

    
	/*
	CImageList flagsUnchecked;
	flagsUnchecked.Create(32, 16, TRUE|ILC_COLOR32, 4, 0);
	CImageList flagsChecked;
	flagsChecked.Create(32, 16, TRUE|ILC_COLOR32, 4, 0);
	CBitmap BTmp1;
	BTmp1.LoadBitmap(IDB_FLAG_TREES);
	flagsUnchecked.Add(&BTmp1, RGB(255,255,255));
	BTmp1.DeleteObject();
	CBitmap BTmp2;
	BTmp2.LoadBitmap(IDB_FLAG_TREES_CHECKED);
	flagsChecked.Add(&BTmp2, RGB(255,255,255));
	BTmp2.DeleteObject();
	int nb = flagsUnchecked.GetImageCount();
	for(int i=0;i<nb;i++)
	{
		m_imageState.Add(flagsUnchecked.ExtractIcon(i));
		m_imageState.Add(flagsChecked.ExtractIcon(i));
	}
	

	CBitmap check;
	check.LoadBitmap(IDB_CHECKBOX);
	m_imageState.Add(&check, RGB(255,255,255));
	check.DeleteObject();
*/


	SetImageList(&m_imageState,TVSIL_NORMAL);
}

/////////////////////////////////////////////////////////////////////////////
// GetSelectedCount
// - returns number of selection tree items

UINT CMultiTree::GetSelectedCount() const
{
	UINT nCount = 0;
	HTREEITEM hItem = GetFirstSelectedItem();
	while (hItem) {
		nCount++;
		hItem = GetNextSelectedItem(hItem);
	}
	return nCount;
}

void CMultiTree::DeleteChildItems(HTREEITEM hParentItem)
{
	// Delete all of the children of hmyItem.
	if (ItemHasChildren(hParentItem))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hParentItem);

		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);
			DeleteItem(hChildItem);
			hChildItem = hNextItem;
		}
	}
}


HTREEITEM CMultiTree::InsertItem(int iIconType, int iFlag, bool Checked,LPCTSTR lpszItem, HTREEITEM hParent)
{
	HTREEITEM ret = NULL;
	if(iIconType<6)
		ret = CTreeCtrl::InsertItem(lpszItem,iIconType,iIconType,hParent);
	if(iIconType==6)
	{
		int icon = iFlag;
		if(Checked) icon++;
        ret = CTreeCtrl::InsertItem(lpszItem,icon,icon,hParent);
	}
	if(iIconType==7)
	{
		int pos = m_imageState.GetImageCount()-2;
		if(pos) pos++;
        ret = CTreeCtrl::InsertItem(lpszItem,pos,pos,hParent);
	}
	if(iIconType==10)
		ret = CTreeCtrl::InsertItem(lpszItem,iIconType,iIconType,hParent);
	return ret;
}

// Inserts a new item to the control.
HTREEITEM CMultiTree::InsertItem(LPTVINSERTSTRUCT lpInsertStruct)
{
	lpInsertStruct->item.iImage = 0;
	lpInsertStruct->item.iSelectedImage = 0;
	 HTREEITEM ret = CTreeCtrl::InsertItem(lpInsertStruct);
	 return ret;
}

HTREEITEM CMultiTree::InsertItem(UINT nMask, LPCTSTR lpszItem, int nImage,
		int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam,
		HTREEITEM hParent, HTREEITEM hInsertAfter)
{
	HTREEITEM ret =  CTreeCtrl::InsertItem( nMask, lpszItem, 0,0, 
							nState, nStateMask, lParam, hParent, hInsertAfter);
	 return ret;
}

HTREEITEM CMultiTree::InsertItem(LPCTSTR lpszItem, HTREEITEM hParent,
					HTREEITEM hInsertAfter)
{
	HTREEITEM ret =  CTreeCtrl::InsertItem(lpszItem,hParent, hInsertAfter);
	 return ret;
}

HTREEITEM CMultiTree::InsertItem(LPCTSTR lpszItem, int nImage, int nSelectedImage,
					HTREEITEM hParent, HTREEITEM hInsertAfter)
{

	HTREEITEM ret =  CTreeCtrl::InsertItem(lpszItem,0,0,
											hParent, hInsertAfter);
	 return ret;
}


/////////////////////////////////////////////////////////////////////////////
// SetMultiSelect
// - allow mode to be turned off

BOOL CMultiTree::SetMultiSelect(BOOL bMulti)
{
	BOOL b = m_bMulti;
	m_bMulti = bMulti;
	/*if (!m_bMulti) {
		while(m_Items.GetCount())
		{
			ITEM_STRUCT_SAVE item = m_Items[0];
			HTREEITEM hTree=InsertItem(item.text,item.hParent);
			m_Items.RemoveAt(0);
		}
		HTREEITEM hItem = GetSelectedItem();
		if (hItem && !IsSelected(hItem))
			hItem = NULL;
		SelectAllIgnore(FALSE, hItem);
		if (hItem)
			SelectItem(hItem);
		RedrawWindow();
	}
	else
	{
		// Delete all of the children of hmyItem.
		HTREEITEM hItem=GetRootItem();
		while(hItem!=NULL)
		{
			if (ItemHasChildren(hItem))
			{
				HTREEITEM hNextItem;
				HTREEITEM hChildItem = GetChildItem(hItem);
				while (hChildItem != NULL)
				{
					hNextItem = GetNextItem(hChildItem, TVGN_NEXT);
					ITEM_STRUCT_SAVE item;
					item.hParent = hItem;
					item.text=GetItemText(hChildItem);
					m_Items.Add(item);
					DeleteItem(hChildItem);
					hChildItem = hNextItem;
				}
			}
			hItem=GetNextItem(hItem,TVGN_NEXT  );
		}
	}*/
	return b;
}

/////////////////////////////////////////////////////////////////////////////
// SetItemState
// - replacement to handle TVIS_FOCUSED

BOOL CMultiTree::SetItemState(HTREEITEM hItem, UINT nState, UINT nStateMask)
{
	ASSERT(hItem);

	if (!m_bMulti)
		return CMultiTree_BASE::SetItemState(hItem, nState, nStateMask);

	HTREEITEM hFocus = GetSelectedItem();		//current focus
	BOOL bWasFocus = (hFocus == hItem);
	BOOL bFocusWasSel = hFocus && IsSelected(hFocus);	//selection state of current focus
	BOOL bWasSel = IsSelected(hItem);		//select state of acting item

	UINT nS = nState & ~TVIS_FOCUSED;
	UINT nSM = nStateMask & ~TVIS_FOCUSED;

	if (nStateMask & TVIS_FOCUSED) {
		//wanted to affect focus
		if (nState & TVIS_FOCUSED) {
			//wanted to set focus
			if (!bWasFocus && bFocusWasSel) {
				//because SelectItem would de-select the current 'real' selection
				// (the one with focus), need to make the tree ctrl think there is
				// no 'real' selection but still keep the the old item selected
				//it has to be done before the SelectItem call because
				// otherwise the TVN_SELCHANGING/ED notification handlers
				// wouldn't be able to get the proper list of selected items
				CMultiTree_BASE::SelectItem(NULL);	//will cause notify, but can be taken as focus change
				CMultiTree_BASE::SetItemState(hFocus, TVIS_SELECTED, TVIS_SELECTED);
				UpdateWindow();
			}
			if (!CMultiTree_BASE::SelectItem(hItem))	//set focus (will consequently select, if not already focused)
				return FALSE;
			if (nStateMask & TVIS_SELECTED) {
				//wanted to affect select state
				if (nState & TVIS_SELECTED) {
					//wanted to select, already done if wasn't focused
					if (!bWasFocus || bFocusWasSel) {
						nS &= ~TVIS_SELECTED;
						nSM &= ~TVIS_SELECTED;
					}
				}
				//else wanted to clear, base call will do that
			}
			else {
				//didn't want to affect select state
				if (!bWasSel) {
					//it wasn't previously selected, let base clear (correct)
					nS &= ~TVIS_SELECTED;
					nSM |= TVIS_SELECTED;
				}
				//else was already selected, no harm done
			}
		}
		else {
			//wanted to clear focus
			if (bWasFocus) {
				//it had the focus
				CMultiTree_BASE::SelectItem(NULL);	//clear focus
				if (!(nStateMask & TVIS_SELECTED)) {
					//didn't want to affect select state
					if (bWasSel) {
						//it was selected, so restore
						ASSERT( !(nS & TVIS_SELECTED) );
						ASSERT( !(nSM & TVIS_SELECTED) );
						//set state here, to avoid double-notify
						CMultiTree_BASE::SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
						//let base do other states
					}
				}
				else if (nState & TVIS_SELECTED) {
					//wanted to select (but clear focus)
					if (bWasSel) {
						//if was selected, restore
						CMultiTree_BASE::SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
					}
					//don't want to notify, default did it
					nS &= ~TVIS_SELECTED;
					nSM &= ~TVIS_SELECTED;
				}
			}
		}
	}

	if (!nSM)
		return TRUE;	//no other states to alter

	if (nSM & TVIS_SELECTED) {
		//still need to alter selection state
		NMTREEVIEW nmtv;
		nmtv.hdr.hwndFrom = m_hWnd;
		nmtv.hdr.idFrom = ::GetDlgCtrlID(m_hWnd);
		nmtv.hdr.code = TVN_SELCHANGING;
		nmtv.itemOld.mask = nmtv.itemNew.mask = 0;
		nmtv.itemOld.hItem = nmtv.itemNew.hItem = NULL;
		TVITEM& item = (nS & TVIS_SELECTED) ? nmtv.itemNew : nmtv.itemOld;
		item.mask = TVIF_HANDLE|TVIF_PARAM;
		item.hItem = hItem;
		item.lParam = GetItemData(hItem);
		if (_SendNotify(&nmtv.hdr))
			return FALSE;	//sel-changing stopped
		VERIFY( CMultiTree_BASE::SetItemState(hItem, nS, nSM) );
		nmtv.hdr.code = TVN_SELCHANGED;
		_SendNotify(&nmtv.hdr);
		nS &= ~TVIS_SELECTED;
		nSM &= ~TVIS_SELECTED;
	}
	if (!nSM)
		return TRUE;
	return CMultiTree_BASE::SetItemState(hItem, nS, nSM);
}

UINT CMultiTree::GetItemState(HTREEITEM hItem, UINT nStateMask) const
{
	UINT n = CMultiTree_BASE::GetItemState(hItem, nStateMask & ~TVIS_FOCUSED);
	if (nStateMask & TVIS_FOCUSED)
		if (GetSelectedItem() == hItem)
			n |= TVIS_FOCUSED;
	return n;
}

BOOL CMultiTree::SelectItem(HTREEITEM hItem)
{
	if (m_bMulti) {
		TRACE(_T("Use SetItemState or FocusItem when in multi-select mode\n"));
		ASSERT(FALSE);
	}
	return CMultiTree_BASE::SelectItem(hItem);
}

BOOL CMultiTree::FocusItem(HTREEITEM hItem)
{
	ASSERT(m_bMulti);

	BOOL bRet = FALSE;
	if (hItem)
		bRet = SetItemState(hItem, TVIS_FOCUSED, TVIS_FOCUSED);
	else {
		hItem = CMultiTree_BASE::GetSelectedItem();
		if (hItem)
			bRet = SetItemState(hItem, 0, TVIS_FOCUSED);
	}
	return bRet;
}

/////////////////////////////////////////////////////////////////////////////
// _SendNotify
// - helper to distinguish between default control generated notifications
//   and this classes emulated ones (so can tell if focus or select notify)

BOOL CMultiTree::_SendNotify(LPNMHDR pNMHDR)
{
	ASSERT(::GetParent(m_hWnd));	//never expected this

	BOOL b = m_bEmulated;
	m_bEmulated = TRUE;
	BOOL bRes = ::SendMessage(::GetParent(m_hWnd), WM_NOTIFY, (WPARAM)pNMHDR->idFrom, (LPARAM)pNMHDR);
	m_bEmulated = b;
	return bRes;
}

/////////////////////////////////////////////////////////////////////////////
// Selection Parsing

HTREEITEM CMultiTree::GetFirstSelectedItem() const
{
	HTREEITEM hItem = GetRootItem();
	while (hItem) {
		if (IsSelected(hItem))
			break;
		hItem = GetNextVisibleItem(hItem);
	}
	return hItem;
}

HTREEITEM CMultiTree::GetNextSelectedItem(HTREEITEM hItem) const
{
	hItem = GetNextVisibleItem(hItem);
	while (hItem) {
		if (IsSelected(hItem))
			break;
		hItem = GetNextVisibleItem(hItem);
	}
	return hItem;
}

void CMultiTree::SelectAll(BOOL bSelect /*= TRUE*/)
{
	bSelect = !!bSelect;	//ensure 0 or 1
	UINT nState = bSelect ? TVIS_SELECTED : 0;
	HTREEITEM hItem = GetRootItem();
	while (hItem) {
		if (IsSelected(hItem) != bSelect)
			SetItemState(hItem, nState, TVIS_SELECTED);
		hItem = GetNextVisibleItem(hItem);
	}
}

void CMultiTree::SelectAllIgnore(BOOL bSelect, HTREEITEM hIgnore)
{
	//special case to avoid multiple notifications for
	// the same item
	bSelect = !!bSelect;	//ensure 0 or 1
	UINT nState = bSelect ? TVIS_SELECTED : 0;
	HTREEITEM hItem = GetRootItem();
	while (hItem) {
		if (hItem != hIgnore)
			if (IsSelected(hItem) != bSelect)
				SetItemState(hItem, nState, TVIS_SELECTED);
		hItem = GetNextVisibleItem(hItem);
	}
}

void CMultiTree::SelectRange(HTREEITEM hFirst, HTREEITEM hLast, BOOL bOnly /*= TRUE*/)
{
	//locate (and select) either first or last
	// (so order is arbitrary)
	HTREEITEM hItem = GetRootItem();
	while (hItem) {
		if ((hItem == hFirst) || (hItem == hLast)) {
			if (hFirst != hLast) {
				if (!IsSelected(hItem))
					SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
				hItem = GetNextVisibleItem(hItem);
			}
			break;
		}

		if (bOnly && IsSelected(hItem))
			SetItemState(hItem, 0, TVIS_SELECTED);
		hItem = GetNextVisibleItem(hItem);
	}
	//select rest of range
	while (hItem) {
		if (!IsSelected(hItem))
			SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
		if ((hItem == hFirst) || (hItem == hLast)) {
			hItem = GetNextVisibleItem(hItem);
			break;
		}
		hItem = GetNextVisibleItem(hItem);
	}
	if (!bOnly)
		return;
	while (hItem) {
		if (IsSelected(hItem))
			SetItemState(hItem, 0, TVIS_SELECTED);
		hItem = GetNextVisibleItem(hItem);
	}
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonDown

#define _bShift	(nFlags & MK_SHIFT)
#define _bCtrl	(nFlags & MK_CONTROL)

CPoint CMultiTree::GetActualHitPoint()
{
	return CPoint(m_xActualHit,m_yActualHit);
}

void CMultiTree::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_xActualHit = point.x;
	m_yActualHit = point.y;

	UINT uFlags=0;
	HTREEITEM hti = HitTest(point,&uFlags);

	int img,im;
	GetItemImage(hti,img,im);
	if( uFlags & TVHT_ONITEMICON)
	{
		if(!m_bMultiCheck)
			UnCheckAllOther(hti);

		int iImg,im;
		GetItemImage(hti,iImg,im);
		im = iImg;
		if(iImg>5 && iImg != 10)
		{
			if(iImg%2) iImg--;
			else iImg++;
			SetItemImage(hti,iImg,iImg);
		}

		/*
		CString str = GetCheckedAudios(GetParentItem(GetParentItem(hti)));
		if(str=="" && im<451)
			SetItemImage(hti,im,im);
		*/
		return;
	}

	OnButtonDown(TRUE, nFlags, point);
}

void CMultiTree::OnRButtonDown(UINT nFlags, CPoint point) 
{
	OnButtonDown(FALSE, nFlags, point);
}

void CMultiTree::OnButtonDown(BOOL bLeft, UINT nFlags, CPoint point)
{
	UINT nHF;
	HTREEITEM hItem;

	BOOL bBase = !m_bMulti;
	if (!bBase) {
		hItem = HitTest(point, &nHF);
		if (hItem) {
			//Versuch
			//(doesn't really mean much to right button, but pass anyway)
			bBase = (nHF & (TVHT_ONITEMBUTTON/*|TVHT_ONITEMINDENT*/));
			if (!bBase && bLeft && (GetStyle() & TVS_CHECKBOXES)) {
				//when the tree has check-boxes, the default handler makes
				// a quick selection of the clicked item, then re-selects
				// the previously selected item - to cause a sel-changed
				// notification.  Fortunately it doesn't affect the multi-
				// selection, so can pass on.
				bBase = (nHF & TVHT_ONITEMSTATEICON);

#ifdef _MST_MULTI_CHECK
				//Use the above define if you want all selected items to
				// be checked the same when any one of them is checked
				// - Interestingly this doesn't happen in the listview control
				//  (LVS_EX_CHECKBOXES)
				if (bBase) {
					//the default selection notification would mess
					// the multi-selection up, so generate the notification
					// manually
					// (anyway, this is smoother than the selection flicker
					//  the default gives)
					NMTREEVIEW nmtv;
#ifdef TVN_CHKCHANGE
					nmtv.hdr.code = TVN_CHKCHANGE;
#else
					nmtv.hdr.code = TVN_SELCHANGED;
#endif
					nmtv.hdr.hwndFrom = m_hWnd;
					nmtv.hdr.idFrom = ::GetDlgCtrlID(m_hWnd);
					nmtv.itemOld.hItem = NULL;
					nmtv.itemNew.mask = TVIF_HANDLE|TVIF_PARAM;

					BOOL bChk = !GetCheck(hItem);
					if (IsSelected(hItem)) {
						HTREEITEM h = GetFirstSelectedItem();
						while (h) {
							if (!GetCheck(h) == bChk) {		//! to ensure 0 or 1
								SetCheck(h, bChk);
#ifdef TVN_CHKCHANGE
								//only send multiple check-change
								// notifications (not sel-changed)
								if (h != hItem) {		//clicked item will be done last
									nmtv.itemNew.hItem = h;
									nmtv.itemNew.lParam = GetItemData(h);
									_SendNotify(&nmtv.hdr);
								}
#endif
							}
							h = GetNextSelectedItem(h);
						}
					}
					else
						SetCheck(hItem, bChk);
					//notify clicked item
					nmtv.itemNew.hItem = hItem;
					nmtv.itemNew.lParam = GetItemData(hItem);
					_SendNotify(&nmtv.hdr);
					return;
				}
#endif

			}
		}
	}
	if (bBase) {
		if (bLeft)
			CMultiTree_BASE::OnLButtonDown(nFlags, point);
		else
			CMultiTree_BASE::OnRButtonDown(nFlags, point);
		return;
	}

	if (!hItem || (nHF & (TVHT_ONITEMRIGHT|TVHT_NOWHERE|TVHT_ONITEMINDENT))) {
		//clicked in space, do rubber-banding
		DoBanding(bLeft, nFlags, point);
		return;
	}

	ASSERT(nHF & (TVHT_ONITEM|TVHT_ONITEMSTATEICON) );	//nothing else left

	DoPreSelection(hItem, bLeft, nFlags);
	DoAction(hItem, bLeft, nFlags, point);
}

void CMultiTree::DoPreSelection(HTREEITEM hItem, BOOL bLeft, UINT nFlags)
{
	if (bLeft) {
		//if shift key down, select immediately
		if (_bShift) {
			if (!m_hSelect)
				m_hSelect = GetSelectedItem();	//focus
			SelectRange(m_hSelect, hItem, !_bCtrl);
			SetItemState(hItem, TVIS_FOCUSED, TVIS_FOCUSED);	//focus changes to last clicked
		}
		else {
			if (!_bCtrl) {
				//if ctrl was down, then the selection is delayed until
				// mouse up, otherwise select the one item
				if (!IsSelected(hItem))
					SelectAllIgnore(FALSE, hItem);
				SetItemState(hItem, TVIS_SELECTED|TVIS_FOCUSED, TVIS_SELECTED|TVIS_FOCUSED);
			}
			m_hSelect = NULL;	//reset when a non-shift operation occurs
		}
		return;
	}

	//right mouse
	if (nFlags & (MK_CONTROL|MK_SHIFT)) {
		if (!_bShift)
			m_hSelect = hItem;
		return;		//do nothing if shift or ctrl
	}
	if (!IsSelected(hItem))
		SelectAllIgnore(FALSE, hItem);
	SetItemState(hItem, TVIS_SELECTED|TVIS_FOCUSED, TVIS_SELECTED|TVIS_FOCUSED);
}

void CMultiTree::DoAction(HTREEITEM hItem, BOOL bLeft, UINT nFlags, CPoint point)
{
	::SetCapture(m_hWnd);
	ASSERT(::GetCapture() == m_hWnd);

	MSG msg;
	UINT nDone = 0;
	CPoint pt;
	CSize sizeDrag(::GetSystemMetrics(SM_CXDRAG), ::GetSystemMetrics(SM_CYDRAG));

	while (!nDone && ::GetMessage(&msg, NULL, 0, 0)) {

		if (::GetCapture() != m_hWnd)
			break;

		switch (msg.message) {
			case WM_MOUSEMOVE:
				pt.x = GET_X_LPARAM(msg.lParam);
				pt.y = GET_Y_LPARAM(msg.lParam);
				if ((abs(pt.x - point.x) > sizeDrag.cx)
								|| ((abs(pt.y - point.y) > sizeDrag.cy)) )
					nDone = 2;
					//because we exit loop, button up will still be dispatched
					// which means WM_CONTEXTMENU will be sent after TVN_BEGINRDRAG
					// - this is the same behaviour as original tree
				break;

			case WM_LBUTTONUP:
			case WM_RBUTTONUP:
				nDone = 1;
				break;

			default:
				::DispatchMessage(&msg);
				break;
		}
	}

	::ReleaseCapture();
	ASSERT(::GetCapture() != m_hWnd);

	//construct tree notification info
	NMTREEVIEW nmtv;
	nmtv.hdr.hwndFrom = m_hWnd;
	nmtv.hdr.idFrom = ::GetDlgCtrlID(m_hWnd);
	nmtv.itemNew.mask = TVIF_HANDLE|TVIF_PARAM;
	nmtv.itemNew.hItem = hItem;
	nmtv.itemNew.lParam = GetItemData(hItem);
	DWORD dwStyle = GetStyle();

	if (nDone == 1) {
		//click
		if (!_bShift && bLeft) {
			UINT nState = TVIS_SELECTED;
			if (_bCtrl)
				nState ^= (GetItemState(hItem, TVIS_SELECTED) & TVIS_SELECTED);
			else
				SelectAllIgnore(FALSE, hItem);
			SetItemState(hItem, TVIS_FOCUSED|nState, TVIS_FOCUSED|TVIS_SELECTED);
		}
		if (::GetFocus() != m_hWnd)
			::SetFocus(m_hWnd);
		nmtv.hdr.code = bLeft ? NM_CLICK : NM_RCLICK;
		_SendNotify(&nmtv.hdr);
	}
	else if (nDone == 2) {
		//drag
		SetItemState(hItem, TVIS_FOCUSED|TVIS_SELECTED, TVIS_FOCUSED|TVIS_SELECTED);
		if (!(dwStyle & TVS_DISABLEDRAGDROP)) {
			nmtv.hdr.code = bLeft ? TVN_BEGINDRAG : TVN_BEGINRDRAG;
			nmtv.ptDrag = point;
			_SendNotify(&nmtv.hdr);
		}
	}
}

void CMultiTree::DoBanding(BOOL bLeft, UINT nFlags, CPoint point)
{
	if (::GetFocus() != m_hWnd)
		::SetFocus(m_hWnd);

	::SetCapture(m_hWnd);

	CTreeItemList list;
	if (nFlags & (MK_SHIFT|MK_CONTROL))
		GetSelectedList(list);

	CClientDC dc(this);
	CRect rectCli;
	GetClientRect(&rectCli);

	MSG msg;
	BOOL bDone = FALSE;
	CPoint pt;
	CSize sizeDrag(::GetSystemMetrics(SM_CXDRAG), ::GetSystemMetrics(SM_CYDRAG));
	BOOL bDrag = FALSE;
	CSize sizeEdge(1, 1);

	UINT nTimer = SetTimer(1, MST_TIMER_PERIOD, NULL);	//for scroll
	CPoint ptScr(GetScrollPos(SB_HORZ), GetScrollPos(SB_VERT));
	CRect rect(0, 0, 0, 0);
	UINT h = 0;
	HTREEITEM hItem = GetRootItem();
	if (hItem) {
		GetItemRect(hItem, &rect, FALSE);
		ptScr.y *= (h = rect.Height());		//this assumes equal height items
	}
	point += ptScr;

	while (!bDone && ::GetMessage(&msg, NULL, 0, 0)) {

		if (::GetCapture() != m_hWnd)
			break;

		switch (msg.message) {
			case WM_TIMER:
				pt = msg.pt;
				ScreenToClient(&pt);
				if (rectCli.PtInRect(pt)) {
					::DispatchMessage(&msg);
					break;
				}
				msg.lParam = MAKELPARAM(pt.x, pt.y);
				//fall through to mousemove

			case WM_MOUSEMOVE:
				pt.x = GET_X_LPARAM(msg.lParam);
				pt.y = GET_Y_LPARAM(msg.lParam);
				if (!bDrag) {
					if ((abs(pt.x - point.x) <= sizeDrag.cx)
							&& ((abs(pt.y - point.y) <= sizeDrag.cy)) )
						break;
					bDrag = TRUE;
					if (!(nFlags & (MK_CONTROL|MK_SHIFT)))
						SelectAll(FALSE);
					UpdateWindow();
					rect.SetRect(point, point);
					dc.DrawDragRect(rect, sizeEdge, NULL, sizeEdge);
				}

				dc.DrawDragRect(rect, sizeEdge, NULL, sizeEdge);	//delete

				if (pt.y < rectCli.top)
					::SendMessage(m_hWnd, WM_VSCROLL, SB_LINEUP, 0);
				else if (pt.y >= rectCli.bottom)
					::SendMessage(m_hWnd, WM_VSCROLL, SB_LINEDOWN, 0);
				if (pt.x < rectCli.left)
					::SendMessage(m_hWnd, WM_HSCROLL, SB_LINELEFT, 0);
				else if (pt.x >= rectCli.right)
					::SendMessage(m_hWnd, WM_HSCROLL, SB_LINERIGHT, 0);

				ptScr = point;
				ptScr.x -= GetScrollPos(SB_HORZ);
				ptScr.y -= GetScrollPos(SB_VERT) * h;
				rect.SetRect(ptScr, pt);
				rect.NormalizeRect();
				UpdateSelectionForRect(rect, nFlags, list);
				dc.DrawDragRect(rect, sizeEdge, NULL, sizeEdge);	//draw
				break;

			case WM_LBUTTONUP:
			case WM_RBUTTONUP:
				bDone = TRUE;
				break;

			case WM_KEYDOWN:
				if (LOWORD(msg.wParam) == VK_ESCAPE) {
					SelectAll(FALSE);
					bDone = TRUE;
					break;
				}
				//dispatch

			default:
				::DispatchMessage(&msg);
				break;
		}
	}
	KillTimer(nTimer);
	::ReleaseCapture();

	if (bDrag)
		dc.DrawDragRect(rect, sizeEdge, NULL, sizeEdge);
	else
		if (!(nFlags & (MK_CONTROL|MK_SHIFT)))
			SelectAll(FALSE);

	//construct notification info
	NMHDR hdr;
	hdr.hwndFrom = m_hWnd;
	hdr.idFrom = ::GetDlgCtrlID(m_hWnd);
	hdr.code = bLeft ? NM_CLICK : NM_RCLICK;
	_SendNotify(&hdr);

	//when banding, make sure we receive WM_CONTEXTMENU
	// every time - which is what the listview ctrl does
	::DispatchMessage(&msg);
}

void CMultiTree::UpdateSelectionForRect(LPCRECT pRect, UINT nFlags, CTreeItemList& list)
{
	CRect rect;
	BOOL bSel;
	POSITION pos;

	HTREEITEM hItem = GetRootItem();
	while (hItem) {
		bSel = IsSelected(hItem);
		GetItemRect(hItem, &rect, m_bBandLabel);
		if (rect.IntersectRect(rect, pRect)) {
			//item in rect
			pos = list.Find(hItem);
			if (!bSel && !pos)
				SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
			else if (_bCtrl && pos)
				SetItemState(hItem, 0, TVIS_SELECTED);
			else if (_bShift && pos)
				list.RemoveAt(pos);		//if shift and in rect, don't lock anymore
		}
		else {
			//item not in rect
			pos = list.Find(hItem);
			if (bSel && !pos)
				SetItemState(hItem, 0, TVIS_SELECTED);
			else if (pos)
				SetItemState(hItem, TVIS_SELECTED, TVIS_SELECTED);
		}
		hItem = GetNextVisibleItem(hItem);
	}
	UpdateWindow();
}

void CMultiTree::OnSetFocus(CWnd* pOldWnd) 
{
	CMultiTree_BASE::OnSetFocus(pOldWnd);
	if (m_bMulti) {
		//'emulated' selected items will remain greyed
		// if application gets covered
		HTREEITEM hItem = GetFirstSelectedItem();
		RECT rect;
		while (hItem) {
			GetItemRect(hItem, &rect, TRUE);
			InvalidateRect(&rect);
			hItem = GetNextSelectedItem(hItem);
		}
	}
}

void CMultiTree::OnKillFocus(CWnd* pNewWnd) 
{
	CMultiTree_BASE::OnKillFocus(pNewWnd);
	if (m_bMulti) {
		//'emulated' selected items may not get
		// refreshed to grey
		HTREEITEM hItem = GetFirstSelectedItem();
		RECT rect;
		while (hItem) {
			GetItemRect(hItem, &rect, TRUE);
			InvalidateRect(&rect);
			hItem = GetNextSelectedItem(hItem);
		}
	}
}

BOOL CMultiTree::OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (!m_bMulti)
		return FALSE;

	LPNMTREEVIEW pNMTreeView = (LPNMTREEVIEW)pNMHDR;
	*pResult = 0;
	if ((pNMTreeView->action == TVE_COLLAPSE) || (pNMTreeView->action == TVE_COLLAPSERESET)) {
		//clear selection of children, it would be confusing otherwise
		// - the notifications can be over-ridden to stop the de-selection
		// if required
		//unfortunately, the parent window can't over-ride this functionality
		// because MFC gives this class first crack.  So if changes are required
		// a derived class will have to be used..
		ASSERT(pNMTreeView->itemNew.hItem);

		//if a descendent item has focus the parent will get selected as a
		// consequence of collapsing the tree, so preserve
		// (if the parent was already selected it will gain focus, but
		//  that's acceptable)
		BOOL bWasSel = IsSelected(pNMTreeView->itemNew.hItem);
		BOOL bWasFocus = SelectChildren(pNMTreeView->itemNew.hItem, FALSE, TRUE);
		if (bWasFocus && !bWasSel)
			CMultiTree_BASE::SelectItem(NULL);	//stop parent from gaining selection
	}

	return FALSE;	//pass to parent
}

BOOL CMultiTree::SelectChildren(HTREEITEM hParent, BOOL bSelect /*= TRUE*/, BOOL bRecurse /*= TRUE*/)
{
	UINT nS = bSelect ? TVIS_SELECTED : 0;

	BOOL bFocusWasInHere = FALSE;

	HTREEITEM hItem = GetNextItem(hParent, TVGN_CHILD);
	while (hItem) {
		UINT nState = GetItemState(hItem, TVIS_SELECTED|TVIS_EXPANDED|TVIS_FOCUSED);
		if ((nState & TVIS_SELECTED) != nS)
			SetItemState(hItem, nS, TVIS_SELECTED);
		bFocusWasInHere |= (nState & TVIS_FOCUSED);
		if (bRecurse && (nState & TVIS_EXPANDED))
			bFocusWasInHere |= SelectChildren(hItem, bSelect, bRecurse);
		hItem = GetNextSiblingItem(hItem);
	}
	return bFocusWasInHere;
}

void CMultiTree::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if( nChar == VK_SPACE )
	{
		HTREEITEM hti = GetSelectedItem();
		if(!m_bMultiCheck)
			UnCheckAllOther(hti);
		
		int iImg,im;
		GetItemImage(hti,iImg,im);
		im = iImg;
		if(iImg>5 && iImg != 10)
		{
			if(iImg%2) iImg--;
			else iImg++;
			SetItemImage(hti,iImg,iImg);
		}

		CString str = GetCheckedAudios(GetParentItem(GetParentItem(hti)));
		if(str=="")
			SetItemImage(hti,im,im);

		return;
	}

	if (!m_bMulti) {
		CMultiTree_BASE::OnKeyDown(nChar, nRepCnt, nFlags);
		return;
	}
		
	BOOL bCtrl = (GetKeyState(VK_CONTROL) & 0x8000);
	BOOL bShift = (GetKeyState(VK_SHIFT) & 0x8000);

	BOOL bDir = FALSE;
	HTREEITEM hSel = NULL;
	switch (nChar) {
		case VK_UP:
			bDir = TRUE;
		case VK_DOWN:
			//common
			hSel = GetSelectedItem();
			if (!m_hSelect)
				m_hSelect = hSel;

			if (!bCtrl && !bShift) {
				m_hSelect = NULL;	//reset
				SelectAll(FALSE);
			}
			break;
//		case VK_SPACE:
//			hSel = GetSelectedItem();
//			if (hSel) {
//				BOOL b = IsSelected(hSel);
//				if (bCtrl)
//					SetItemState(hSel, b ? 0 : TVIS_SELECTED, TVIS_SELECTED);
//				else if (!b)
//					SetItemState(hSel, TVIS_SELECTED, TVIS_SELECTED);
//			}
//			return;		//don't call base class (it would start a search on the char)
//TODO: the text search isn't stopped by this ^.  It may be done in the TranslateMessage,
// so would have to trap PreTranslateMessage to avoid it.  If 'space' selection is
// required then need to implement - I'm not bothered.
	}

	CMultiTree_BASE::OnKeyDown(nChar, nRepCnt, nFlags);
	if (!hSel || (!bCtrl && !bShift) )
		return;

	HTREEITEM hNext = bDir ? GetPrevVisibleItem(hSel) : GetNextVisibleItem(hSel);
	if (!hNext)
		hNext = hSel;
	if (bShift)
		SelectRange(m_hSelect, hNext, TRUE);
	else if (bCtrl)
		SetItemState(hNext, TVIS_FOCUSED, TVIS_FOCUSED);
}

void CMultiTree::GetSelectedList(CTreeItemList& list) const
{
	list.RemoveAll();

	HTREEITEM hItem = GetFirstSelectedItem();
	while (hItem) {
		list.AddTail(hItem);
		hItem = GetNextSelectedItem(hItem);
	}
}

int CMultiTree::GetChildItems(HTREEITEM *hChildItems,HTREEITEM hParent)
{
	int nbItems = 0;
	if (ItemHasChildren(hParent))
	{
		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hParent);

		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);
			hChildItems[nbItems] = hChildItem;
			nbItems++;
			hChildItem = hNextItem;
		}
	}
	return nbItems;
}

void CMultiTree::UnCheckAllOther(HTREEITEM _hItem)
{
	HTREEITEM hItems[32];
	HTREEITEM hParent = GetParentItem(_hItem);
	int nb = GetChildItems(hItems,hParent);
	for(int i=0;i<nb;i++)
	{
		if(hItems[i] != _hItem)
		{
				int iImg,im;
				GetItemImage(hItems[i],iImg,im);
				if(iImg>2)
				{
					if(iImg%2);
					else iImg--;
					SetItemImage(hItems[i],iImg,iImg);
				}
		}
	}
}

HTREEITEM CMultiTree::FindItem(CString str,HTREEITEM hItem)
{
	while(hItem)
	{
		HTREEITEM oldItem = hItem;
		while (hItem) 
		{
			CString s = GetItemText(hItem);
			if(s == str) return hItem;
			hItem = GetNextItem(hItem, TVGN_NEXT);
		}
		hItem = GetChildItem(oldItem);
	}
	return NULL;
}

void CMultiTree::SetCheckedAudio(int n,HTREEITEM hParent)
{
	HTREEITEM hI = FindItem(m_audID,hParent);
	HTREEITEM hItem[32];
	int nb =GetChildItems(hItem,hI);
	SetChecked(hItem[n]);	
}

int CMultiTree::GetCheckedSubs(HTREEITEM hParent)
{


	CString s = GetItemText(hParent);
	if(s!=m_audID && s!=m_subID)
	{
		HTREEITEM hGetParent = GetParentItem(hParent);
		s = GetItemText(hGetParent);
		if(s!=m_audID && s!=m_subID)
			hParent = hParent;
		else hParent = hGetParent;
	}
	else hParent = hParent;

	HTREEITEM hI = FindItem(m_subID,hParent);
	HTREEITEM hChields[32];
	int nb=GetChildItems(hChields,hI);

	int wahl;
	CString str, strID;
	//## COLLECT all checked items in a single string
	long nCount = 0;
	for(long i=0; i<nb; i++)
	{
		if(IsItemChecked(hChields[i]))
		{
			wahl = i;
			return wahl;
		}
	}


	return 99;
}

CString CMultiTree::GetUnCheckedSubIDs(HTREEITEM hParent)
{


	CString s = GetItemText(hParent);
	if(s!=m_audID && s!=m_subID)
	{
		HTREEITEM hGetParent = GetParentItem(hParent);
		s = GetItemText(hGetParent);
		if(s!=m_audID && s!=m_subID)
			hParent = hParent;
		else hParent = hGetParent;
	}
	else hParent = hParent;

	HTREEITEM hI = FindItem(m_subID,hParent);
	HTREEITEM hChields[32];
	int nb=GetChildItems(hChields,hI);

	CString str, strID;
	//## COLLECT all checked items in a single string
	long nCount = 0;
	for(long i=0; i<nb; i++)
	{
		if(!IsItemChecked(hChields[i]))
		{
			strID.Format(TEXT("%d"), GetItemData(hChields[i]));
			str += strID + _T(", ");
			nCount++;
		}
	}

	//## ADJUST String
	if (str.GetLength() > 2) str = str.Mid(0, str.GetLength()-2);
	if (nCount > 0) str = (CString)_T("(") + str + _T(")");

	return str;
}


CString CMultiTree::GetUnCheckedAudioIDs(HTREEITEM hParent)
{


	CString s = GetItemText(hParent);
	if(s!=m_audID && s!=m_subID)
	{
		HTREEITEM hGetParent = GetParentItem(hParent);
		s = GetItemText(hGetParent);
		if(s!=m_audID && s!=m_subID)
			hParent = hParent;
		else hParent = hGetParent;
	}
	else hParent = hParent;

	HTREEITEM hI = FindItem(m_audID,hParent);
	HTREEITEM hChields[32];
	int nb=GetChildItems(hChields,hI);

	CString str, strID;
	//## COLLECT all checked items in a single string
	long nCount = 0;
	for(long i=0; i<nb; i++)
	{
		if(!IsItemChecked(hChields[i]))
		{
			strID.Format(TEXT("%d"), GetItemData(hChields[i]));
			str += strID + _T(", ");
			nCount++;
		}
	}

	//## ADJUST String
	if (str.GetLength() > 2) str = str.Mid(0, str.GetLength()-2);
	if (nCount > 0) str = (CString)_T("(") + str + _T(")");

	return str;
}


CString CMultiTree::GetCheckedAudios(HTREEITEM hParent)
{


	CString s = GetItemText(hParent);
	if(s!=m_audID && s!=m_subID)
	{
		HTREEITEM hGetParent = GetParentItem(hParent);
		s = GetItemText(hGetParent);
		if(s!=m_audID && s!=m_subID)
			hParent = hParent;
		else hParent = hGetParent;
	}
	else hParent = hParent;

	HTREEITEM hI = FindItem(m_audID,hParent);
	HTREEITEM hChields[32];
	int nb=GetChildItems(hChields,hI);

	CString str, strID;
	//## COLLECT all checked items in a single string
	long nCount = 0;
	for(long i=0; i<nb; i++)
	{
		if(IsItemChecked(hChields[i]))
		{
			strID.Format(TEXT("%d"), i);
			str += strID + _T(", ");
			nCount++;
		}
	}

	//## ADJUST String
	if (str.GetLength() > 2) str = str.Mid(0, str.GetLength()-2);
	if (nCount > 0) str = (CString)_T("(") + str + _T(")");

	return str;
}

void CMultiTree::SetCheckedSubtitle(int n,HTREEITEM hParent)
{
	HTREEITEM hI = FindItem(m_subID,hParent);
	HTREEITEM hItem = GetNextVisibleItem(hI);
	int i=0;
	while (GetParentItem(hItem) == hI) 
	{
		if(i==n)
			SetChecked(hItem);
		i++;
		hItem = GetNextVisibleItem(hItem);
	}
}

void CMultiTree::UnCheckAll()
{
	HTREEITEM hItem = GetRootItem();
	while (hItem) 
	{
		UINT im=GetItemState(hItem,TVIS_STATEIMAGEMASK);
		long k=im>>12;
		if(k) 
		{
			k=1;
			SetItemState( hItem, INDEXTOSTATEIMAGEMASK(k), 
						TVIS_STATEIMAGEMASK );
		}
		hItem = GetNextVisibleItem(hItem);
	}
}

void CMultiTree::SetChecked(HTREEITEM hItem)
{
	if(!m_bMultiCheck)
		UnCheckAllOther(hItem);

	int iImg,im;
	GetItemImage(hItem,iImg,im);
	if(iImg>2)
	{
		if(iImg%2)iImg++;
		SetItemImage(hItem,iImg,iImg);
	}

}
void CMultiTree::EnableMultiCheck(bool bEnable)
{
	m_bMultiCheck = bEnable;
}

BOOL CMultiTree::IsItemChecked(HTREEITEM hItem)
{
	BOOL res;
	int iImg,im;
	GetItemImage(hItem,iImg,im);
	if(iImg==6 || iImg == 8)
		res = FALSE;
	else 
		res = TRUE;

	return res;
}

HTREEITEM CMultiTree::GetFirstCheckedItem()
{
	for ( HTREEITEM hItem = GetRootItem(); hItem!=NULL; hItem = GetNextItem( hItem,TVGN_NEXT ) )
		if ( IsItemChecked(hItem) )
			return hItem;

	return NULL;
}

HTREEITEM CMultiTree::GetNextCheckedItem( HTREEITEM hItem )
{
	for ( hItem = GetNextItem( hItem,TVGN_NEXT ); hItem!=NULL; hItem = GetNextItem( hItem,TVGN_NEXT ) )
		if ( IsItemChecked(hItem) )
			return hItem;

	return NULL;
}

HTREEITEM CMultiTree::GetPrevCheckedItem( HTREEITEM hItem )
{
	for ( hItem = GetPrevSiblingItem( hItem ); hItem!=NULL; hItem = GetPrevSiblingItem( hItem ) )
		if ( IsItemChecked(hItem) )
			return hItem;

	return NULL;
}

void CMultiTree::SaveTree(int ID)
{
	m_TreeSave[ID].ID = 1;
	Serialize(true,&m_TreeSave[ID]);
}

bool CMultiTree::LoadTree(int ID)
{
	if(m_TreeSave[ID].ID)
	{
		Serialize(false,&m_TreeSave[ID]);
		return true;
	}
	return false;
}

void CMultiTree::Serialize(bool bSave, TREE_SAVE_DATA *tree) 
{

	if (bSave)
	{	
		tree->data.Reset();
		// storing code
		HTREEITEM hti = GetRootItem();
		while( hti )
		{
			int indent = GetIndentLevel( hti );
			char c[2];
			c[0] = 0x0a; c[1] = 0x00;
			while( indent-- )
				tree->data.WriteString(  _T("\t") );
			int iImage,im;
			GetItemImage(hti,iImage,im);
			int exp=0;
			if(GetItemState( hti,TVIF_STATE)&TVIS_EXPANDED)
				exp = 1;
			TCHAR state[32];
			swprintf(state,_T("%04d%d"),iImage,exp);
			tree->data.WriteString( GetItemText( hti ) + state + _T("\r\n"));
			hti = GetNextItem( hti );
		}
		
	}
	else
	{	
		CArray<HTREEITEM> ExpItems;
		DeleteAllItems();
		// loading code
		CString sLine;
		//tree->data.ResetRdPtr();
		if( !tree->data.ReadString( sLine ) )
			return;

		HTREEITEM hti = NULL;
		int indent, baseindent = 0;
		while( sLine[baseindent] == 0x0a )
			baseindent++;
		do
		{
			if( sLine.GetLength() == 0 )
				continue;
			for( indent = 0; sLine[indent] == '\t'; indent++ )
				;		// We don't need a body
			sLine = sLine.Right( sLine.GetLength() - indent );
			indent -= baseindent;

			HTREEITEM parent;
			int previndent = GetIndentLevel( hti );
			if( indent ==  previndent)
				parent = GetParentItem( hti );
			else if( indent > previndent )
				parent = hti;
			else
			{
				int nLevelsUp = previndent - indent;
				parent = GetParentItem( hti );
				while( nLevelsUp-- )
					parent = GetParentItem( parent );
			}
			//int iImage = _wtoi(sLine.Mid(sLine.GetLength()-6,4));
			int iImage = _wtoi(sLine.Mid(sLine.GetLength()-6,4));
			TCHAR state[10];
			state[1] = '\0';
			state[0]= sLine.GetAt(sLine.GetLength()-2);
			int iExp = _wtoi(state);
			sLine.Delete(sLine.GetLength()-6,6);
			hti = CTreeCtrl::InsertItem( sLine,  iImage,iImage, parent ? parent : TVI_ROOT);
			if(iExp)
				ExpItems.Add(hti);
		}while( tree->data.ReadString( sLine ) );
		//tree->data.Reset();
		for(int x=0;x<ExpItems.GetCount();x++)
			Expand(ExpItems.GetAt(x),TVE_EXPAND);
	}

}

BOOL CMultiTree::DeleteAllItems()
{
	return CTreeCtrl::DeleteAllItems();
}



int CMultiTree::GetIndentLevel( HTREEITEM hItem )
{
	int iIndent = 0;

	while( (hItem = GetParentItem( hItem )) != NULL )
		iIndent++;
	return iIndent;
}

HTREEITEM CMultiTree::GetNextItem( HTREEITEM hItem )
{
	HTREEITEM	hti;

	if( ItemHasChildren( hItem ) )
		return GetChildItem( hItem );		// return first child
	else{
		// return next sibling item
		// Go up the tree to find a parent's sibling if needed.
		while( (hti = GetNextSiblingItem( hItem )) == NULL ){
			if( (hItem = GetParentItem( hItem ) ) == NULL )
				return NULL;
		}
	}
	return hti;
}

HTREEITEM CMultiTree::GetNextItem( HTREEITEM hItem, UINT mask )
{
	return CTreeCtrl::GetNextItem(hItem,mask);
}


void CMultiTree::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CTreeCtrl::OnLButtonDblClk(nFlags, point);
}
