/*****************************************************************************
 * MltiTree.h: ShrinkTo5 Track Selection TreeView class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#if !defined(AFX_MLTITREE_H__9F4750E5_C512_11D2_880A_02608C7CB3EE__INCLUDED_)
#define AFX_MLTITREE_H__9F4750E5_C512_11D2_880A_02608C7CB3EE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// MltiTree.h : header file
// Copyright (c) 1999 Richard Hazlewood
// This code is provided as-is.  Use at your own peril.

#include <afxtempl.h>

/////////////////////////////////////////////////////////////////////////////

//take the liberty of using 1 for focused item - it used
// to be defined anyway, before MS removed it.
#ifndef TVIS_FOCUSED
#define TVIS_FOCUSED	1
#else
#if TVIS_FOCUSED != 1
#error TVIS_FOCUSED was assumed to be 1
#endif
#endif

//define for custom notification
// sent on check-box change
// (Care: may clash in future)
#ifdef _MST_TVN_CHKCHANGE
#ifndef _MST_MULTI_CHECK
#define _MST_MULTI_CHECK	//implied
#endif
#ifndef TVN_CHKCHANGE
#define TVN_CHKCHANGEA	(TVN_FIRST-49)
#define TVN_CHKCHANGEW	(TVN_FIRST-(49+49))
#ifdef UNICODE
#define TVN_CHKCHANGE	TVN_CHKCHANGEW
#else
#define TVN_CHKCHANGE	TVN_CHKCHANGEA
#endif
#endif //!TVN_CHKCHANGE
#endif //_MST_TVN_CHKCHANGE

typedef CTypedPtrList<CPtrList, HTREEITEM> CTreeItemList;

/////////////////////////////////////////////////////////////////////////////
// CMultiTree window

#ifndef MST_AFX_EXT_CLASS
#define MST_AFX_EXT_CLASS
//define this if building class for DLL export/import
// typically: #define MST_AFX_EXT_CLASS AFX_EXT_CLASS
#endif

#ifndef CMultiTree_BASE
#define CMultiTree_BASE CTreeCtrl
#endif

typedef struct ITEM_STRUCT_SAVE
{
	HTREEITEM hParent;
	CString text;
} ITEM_STRUCT_SAVE;

class CTreeSave{
	CString *m_buffer;
	int m_RdPtr;
public:
	CTreeSave(){m_RdPtr = 0;m_buffer = new CString;};
	~CTreeSave(){delete m_buffer;};
	void Reset(){m_RdPtr = 0;delete m_buffer;m_buffer=new CString;};
	void ResetRdPtr(){m_RdPtr = 0;};
	BOOL ReadString(CString& str){
		int f = m_buffer->Find(_T("\r\n"),m_RdPtr);
		if(f==-1)return FALSE;
		str = m_buffer->Mid(m_RdPtr,f-m_RdPtr+1);
		m_RdPtr += str.GetLength()+1;
		return str.GetLength();
	};
	void WriteString(LPCTSTR  str){
		*m_buffer += str;
	};
};

typedef struct TREE_SAVE_DATA
{
	int ID;
	CTreeSave data;
}TREE_SAVE_DATA;

class MST_AFX_EXT_CLASS CMultiTree : public CTreeCtrl
{
	DECLARE_DYNAMIC(CMultiTree)
// Construction
public:
	CMultiTree();

// Attributes
public:

// Operations
public:
	void InitBitmaps();
	HTREEITEM FindItem(CString str,HTREEITEM hParent);
	// Inserts a new item to the control.

	// Tpyes:
	// 0 = DVD icon
	// 1 = Audio icon
	// 2 = Subtitle icon
	// 3 = Flag
	// 4 = nur checkbox
	HTREEITEM InsertItem(int iIconType, int iFlag, bool Checked,LPCTSTR lpszItem, HTREEITEM hParent = TVI_ROOT);
	HTREEITEM InsertItem(LPTVINSERTSTRUCT lpInsertStruct);
	HTREEITEM InsertItem(UINT nMask, LPCTSTR lpszItem, int nImage,
		int nSelectedImage, UINT nState, UINT nStateMask, LPARAM lParam,
		HTREEITEM hParent, HTREEITEM hInsertAfter);
	HTREEITEM InsertItem(LPCTSTR lpszItem, HTREEITEM hParent = TVI_ROOT,
		HTREEITEM hInsertAfter = TVI_LAST);
	HTREEITEM InsertItem(LPCTSTR lpszItem, int nImage, int nSelectedImage,
		HTREEITEM hParent = TVI_ROOT, HTREEITEM hInsertAfter = TVI_LAST);

	// CheckBox helpers
	void SaveTree(int ID);
	bool LoadTree(int ID);

	HTREEITEM GetNextItem( HTREEITEM hItem, UINT mask );

	void SetChecked(HTREEITEM hItm);
	CString GetCheckedAudios(HTREEITEM hParent);
	int GetCheckedSubs(HTREEITEM hParent);
	CString GetUnCheckedAudioIDs(HTREEITEM hParent);
	CString GetUnCheckedSubIDs(HTREEITEM hParent);
	void SetCheckedAudio(int n,HTREEITEM hParent);
	void SetCheckedSubtitle(int n,HTREEITEM hParent);
	void UnCheckAll();
	void UnCheckAllOther(HTREEITEM hItem);
	void EnableMultiCheck(bool bEnable);
	CPoint GetActualHitPoint();
	BOOL IsItemChecked(HTREEITEM hItem);
	HTREEITEM GetFirstCheckedItem();
	HTREEITEM GetNextCheckedItem( HTREEITEM hItem );
	HTREEITEM GetPrevCheckedItem( HTREEITEM hItem );
	BOOL DeleteAllItems();

	UINT GetSelectedCount() const;
	HTREEITEM GetFirstSelectedItem() const;
	HTREEITEM GetNextSelectedItem(HTREEITEM hItem) const;
	
	void SelectAll(BOOL bSelect = TRUE);	//doesn't affect focus
	void SelectRange(HTREEITEM hFirst, HTREEITEM hLast, BOOL bOnly = TRUE);
	BOOL IsSelected(HTREEITEM hItem) const {return !!(TVIS_SELECTED & CTreeCtrl::GetItemState(hItem, TVIS_SELECTED));};

	//SelectChildren returns TRUE if focus was on a child item
	BOOL SelectChildren(HTREEITEM hParent, BOOL bSelect = TRUE, BOOL bRecurse = TRUE);

	//Note: not virtual, so must call this class (or derivative)
	BOOL SetItemState(HTREEITEM hItem, UINT nState, UINT nStateMask);
	UINT GetItemState(HTREEITEM hItem, UINT nStateMask) const;
	BOOL SelectItem(HTREEITEM hItem);

	//Note: GetSelectedItem should be taken as GetFocusedItem
	// when in multi-select mode
	HTREEITEM GetFocusedItem() const {ASSERT(m_bMulti);return CTreeCtrl::GetSelectedItem();};
	BOOL FocusItem(HTREEITEM hItem);

	BOOL SetMultiSelect(BOOL bMulti);
	BOOL IsMultiSelect() const {return m_bMulti;};
	BOOL IsEmulatedNotify() const {return m_bEmulated;};
	BOOL SetBandingHit(BOOL bLabel) {BOOL b=m_bBandLabel;m_bBandLabel=bLabel;return b;};

	void GetSelectedList(CTreeItemList& list) const;

	void DeleteChildItems(HTREEITEM hParentItem);
protected:
	bool		m_bMultiCheck;
	int			m_xActualHit;
	int			m_yActualHit;
	BOOL		m_bMulti;
	CString		m_audID;
	CString		m_subID;
	HTREEITEM	m_hSelect;	//for shift selection
	BOOL		m_bEmulated;
	BOOL		m_bBandLabel;
	BOOL		m_bMultiselect;
	CImageList  m_imageState;
	CArray<ITEM_STRUCT_SAVE>      m_Items;
	TREE_SAVE_DATA m_TreeSave[4];

protected:
	void OnButtonDown(BOOL bLeft, UINT nFlags, CPoint point);
	void DoPreSelection(HTREEITEM hItem, BOOL bLeft, UINT nFlags);
	void DoAction(HTREEITEM hItem, BOOL bLeft, UINT nFlags, CPoint point);
	void DoBanding(BOOL bLeft, UINT nFlags, CPoint point);
	void UpdateSelectionForRect(LPCRECT pRect, UINT nFlags, CTreeItemList& list);
	void SelectAllIgnore(BOOL bSelect, HTREEITEM hIgnore);

	int GetChildItems(HTREEITEM *hChildItems,HTREEITEM hParent);
	HTREEITEM GetNextItem( HTREEITEM hItem );
	int GetIndentLevel( HTREEITEM hItem );
	void Serialize(bool bSave, TREE_SAVE_DATA *tree);

	BOOL _SendNotify(LPNMHDR pNMHDR);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMultiTree)
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMultiTree();

	// Generated message map functions
protected:
	//{{AFX_MSG(CMultiTree)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg BOOL OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MLTITREE_H__9F4750E5_C512_11D2_880A_02608C7CB3EE__INCLUDED_)
