/*****************************************************************************
 * ShrinkTo5GuiDlg.cpp: ShrinkTo5 GUI class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "stdafx.h"
#include "ShrinkTo5Gui.h"
#include "ShrinkTo5GuiDlg.h"
#include ".\shrinkto5guidlg.h"
#include "UnicodeHelpers.h"
#include "MultiLanguage.h"

#include "Donate.h"
#include <dbt.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CShrinkTo5GuiDlg::CShrinkTo5GuiDlg(CWnd* pParent /*=NULL*/)
	: CBDialog(CShrinkTo5GuiDlg::IDD, pParent)
{
	LangID="english";
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CShrinkTo5GuiDlg::DoDataExchange(CDataExchange* pDX)
{
	CBDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_MLANG, m_bLang);
	DDX_Control(pDX, IDC_CBSHRINK, xShrink);
	DDX_Control(pDX, IDC_CBBURN, xBurn);
}

BEGIN_MESSAGE_MAP(CShrinkTo5GuiDlg, CBDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_MLANG, OnBnClickedMlang)
	ON_BN_CLICKED(IDC_CBSHRINK, OnBnClickedCbshrink)
	ON_BN_CLICKED(IDC_CBBURN, OnBnClickedCbburn)
END_MESSAGE_MAP()

/*****************************************************************************
* Create the dialog and fill/load all settings
*****************************************************************************/
BOOL CShrinkTo5GuiDlg::OnInitDialog()
{
	CBDialog::OnInitDialog();

	// Symbol fr dieses Dialogfeld festlegen. Wird automatisch erledigt
	//  wenn das Hauptfenster der Anwendung kein Dialogfeld ist
	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden

	//Get actual Path and load the language file
	int Pos = 0;
	long lPos;
	::GetModuleFileName(AfxGetApp()->m_hInstance, EncLangFile.GetBuffer(MAX_PATH), MAX_PATH);
	EncLangFile.ReleaseBuffer();
	lPos = EncLangFile.ReverseFind('\\');
	if (lPos !=-1)
		EncLangFile = EncLangFile.Left(lPos+1);	
	EncLangFile += "language.ini";

	TCHAR chText[1000];
	if(GetPrivateProfileString(_T("Languages"), _T("default"), _T("#####"), chText, 1000, EncLangFile)){
		GetPrivateProfileString(_T("Languages"), CString(chText), _T("#####"), chText, 1000, EncLangFile);
			LangID=CString(chText);
	}

	VERIFY(drawArea1.Create(CDoShrink::IDD, this));
	WINDOWPLACEMENT WPTemp;
	WPTemp.length = sizeof(WINDOWPLACEMENT);
	GetDlgItem(IDC_TRAGER)->GetWindowPlacement(&WPTemp);
	drawArea1.SetWindowPlacement(&WPTemp);

	drawArea1.ShowWindow(SW_SHOW);

	//Fill the Dialog with Language dependet String
	FillMultiLangGui();
	//CheckDonate();

	m_bLang.SetSkin(IDB_BLANG, 1, 1, 1, 0, 0, 0, 0);
	xShrink.SetSkin(IDB_RSHRINK, 0, 1, 0, 0, 0, 0, 0);
	xBurn.SetSkin(IDB_RBURN, 0, 1, 0, 0, 0, 0, 0);
	xShrink.SetCheck(1);
	xShrink.EnableWindow(FALSE);

	//Build Dialog With Background Bitmap
	SetBitmapStyle(StyleCenter);
	SetBitmap(IDB_MAINDIALOG);

	//Create and attach Tooltips
	if(m_ToolTip.Create(this))
	{
		m_ToolTip.SetDelayTime(TTDT_INITIAL,10);
		m_ToolTip.AddTool(&m_bLang,GetText(TOOL_6));
		//m_ToolTip.AddTool(&bSetBurn,GetText(BURN_1));
		m_ToolTip.Activate(TRUE);
	}

	return TRUE;  // Geben Sie TRUE zurck, auer ein Steuerelement soll den Fokus erhalten
}

// Wenn Sie dem Dialogfeld eine Schaltflche "Minimieren" hinzufgen, bentigen Sie 
//  den nachstehenden Code, um das Symbol zu zeichnen. Fr MFC-Anwendungen, die das 
//  Dokument/Ansicht-Modell verwenden, wird dies automatisch ausgefhrt.

void CShrinkTo5GuiDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext zum Zeichnen

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Symbol in Clientrechteck zentrieren
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Symbol zeichnen
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// Die System ruft diese Funktion auf, um den Cursor abzufragen, der angezeigt wird, whrend der Benutzer
//  das minimierte Fenster mit der Maus zieht.
HCURSOR CShrinkTo5GuiDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

BOOL CShrinkTo5GuiDlg::PreTranslateMessage(MSG* pMsg)
{
	m_ToolTip.RelayEvent(pMsg);

	return CBDialog::PreTranslateMessage(pMsg);
}
/*****************************************************************************
 * FillMultiLangGui
 * Set the new Static Text
 *****************************************************************************/
void CShrinkTo5GuiDlg::FillMultiLangGui()
{
	CString head1;
	head1 = _T("ShrinkTo5Gui - ");
	head1 += GetText(GUI_3);
	this->SetWindowText(head1);
}

/*****************************************************************************
 * OnBnClickedMlang
 * Opens the Language Selectio nDialog. After choosing new languages we will
 * attach the strings completely new.
 *****************************************************************************/
void CShrinkTo5GuiDlg::OnBnClickedMlang()
{
	CMultiLanguage dlg;
	INT_PTR nResponse = dlg.DoModal();
	if(nResponse== IDOK){
		FillMultiLangGui();
		drawArea1.smallWindow.FillMultiLangGui();
		drawArea1.FillMultiLangGui();
		if(drawArea2.IsWindowVisible())
			drawArea2.FillMultiLangGui();
		m_ToolTip.AddTool(&m_bLang,GetText(TOOL_6));
		RedrawWindow();
	}
}

/*****************************************************************************
* WinProc is used to catch minimized/restore commands to control the 
* Workwindow
*****************************************************************************/
LRESULT CShrinkTo5GuiDlg::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	if(message == WM_SIZE)
	{
		if(wParam == SIZE_MINIMIZED)
		{
			drawArea1.smallWindow.ShowWindow(SW_SHOW);
		}
		if(wParam == SIZE_RESTORED){
			drawArea1.smallWindow.ShowWindow(SW_HIDE);
		}

	}
	return CBDialog::WindowProc(message, wParam, lParam);
}

void CShrinkTo5GuiDlg::OnBnClickedCbshrink()
{
	//Makes sure that the Burn Process is not on work. 
	//This avoid bad crashes
	if(drawArea2.onWork==true){
		xShrink.SetCheck(0);
		xBurn.SetCheck(1);
		return;
	}

	//Ok! No Work in progress
	xBurn.SetCheck(0);
	xBurn.EnableWindow(TRUE);
	xShrink.SetCheck(1);
	xShrink.EnableWindow(FALSE);
	
	drawArea2.DestroyWindow();
	drawArea1.ShowWindow(SW_SHOW);	
		
}

void CShrinkTo5GuiDlg::OnBnClickedCbburn()
{

	//Makes sure that the Shrink Process is not on work. 
	//This avoid bad crashes
	if(drawArea1.onWork==true){
		xShrink.SetCheck(1);
		xBurn.SetCheck(0);
		return;
	}

	//Ok! No Work in progress
	HMODULE x = LoadLibrary(_T("FoxBurnerU.dll"));
	if(!x){
		xBurn.SetCheck(0);
		CDonate dlg;
		dlg.DoModal();
		return;
	}else{
		SetCapture();
		BeginWaitCursor();
		xShrink.SetCheck(0);
		xShrink.EnableWindow(TRUE);
		
		xBurn.EnableWindow(FALSE);

		VERIFY(drawArea2.Create(CBurnPlugin::IDD, this));

		WINDOWPLACEMENT WPTemp;
		WPTemp.length = sizeof(WINDOWPLACEMENT);
		GetDlgItem(IDC_TRAGER)->GetWindowPlacement(&WPTemp);
		drawArea2.SetWindowPlacement(&WPTemp);
		drawArea1.ShowWindow(SW_HIDE);
		drawArea2.ShowWindow(SW_SHOW);
		ReleaseCapture();
		EndWaitCursor();
	}

}
